@@ -1,3 +1,3 @@
 from mcf.mcf_functions import modified_causal_forest
 from mcf.optp_functions import optpoltree
-__version__ = '0.2.6'
+from mcf.mcf_functions import ModifiedCausalForest
