@@ -62,7 +62,7 @@ class OutputTerminalFile():
 
     def __init__(self, file_name):
         self.terminal = sys.stdout
-        self.output = open(file_name, "a")
+        self.output = open(file_name, mode="a", encoding="utf-8")
 
     def write(self, message):
         """Write both into terminal and file."""
@@ -195,7 +195,7 @@ def print_effect(est, stderr, t_val, p_val, effect_list, add_title=None,
         if add_title is not None:
             print_str += 'f{add_info:6.2f} '
         print_str += f'{est[j]:12.6f}  {stderr[j]:12.6f} '
-        print_str += f'{t_val[j]:8.2f}  {p_val[j]*100:8.3f}% '
+        print_str += f'{t_val[j]:8.2f}  {p_val[j]:8.3%} '
         if p_val[j] < 0.001:
             print_str += '****'
         elif p_val[j] < 0.01:
@@ -355,8 +355,8 @@ def substitute_variable_name(v_dict, old_name, new_name):
     for i in v_dict:
         list_of_this_dic = v_dict[i]
         if list_of_this_dic is not None:
-            for j, _ in enumerate(list_of_this_dic):
-                if list_of_this_dic[j] == old_name:
+            for j, elem in enumerate(list_of_this_dic):
+                if elem == old_name:
                     list_of_this_dic[j] = new_name
             vn_dict[i] = list_of_this_dic
     return vn_dict
@@ -650,7 +650,7 @@ def add_var_names(names1, names2=None, names3=None, names4=None, names5=None,
 
 
 def print_descriptive_stats_file(indata, varnames='all', to_file=False,
-                                 df_instead_of_file=False):
+                                 df_instead_of_file=False, file_to_print=None):
     """Print descriptive statistics of a dataset on file.
 
     Parameters
@@ -685,6 +685,9 @@ def print_descriptive_stats_file(indata, varnames='all', to_file=False,
                            'display.expand_frame_repr', expand,
                            'chop_threshold', 1e-13):
         print(to_print)
+        if file_to_print is not None:
+            print_f(file_to_print, '\nData set:', indata)
+            print_f(file_to_print, to_print)
 
 
 def check_all_vars_in_data(indata, variables):
@@ -991,7 +994,7 @@ def dic_get_list_of_key_by_item(dic, value):
     for keys in dic.keys():
         if dic[keys] in value:
             key_list += [keys]
-    if key_list == []:
+    if not key_list:
         raise Exception('Retrieving items from list was not succesful')
     return key_list
 
@@ -1095,12 +1098,10 @@ def print_f(file_to_print_to, *strings):
     None.
 
     """
-    with open(file_to_print_to, "a") as file:
+    with open(file_to_print_to, mode="a", encoding="utf-8") as file:
         file.write('\n')
         for text in strings:
             if not isinstance(text, str):
                 file.write(str(text))
             else:
                 file.write(text)
-
-    
\ No newline at end of file
