@@ -38,7 +38,7 @@ def ate_est(weights, pred_data, y_dat, cl_dat, w_dat, var, con,
     balancing_test: Boolean.
               Default is False.
     w_ate_only : Boolean.
-              Only weights are needede as output. Default is False.
+              Only weights are needed as output. Default is False.
     print_output : Boolean.
               Prints output. Default is True.
 
@@ -70,7 +70,7 @@ def ate_est(weights, pred_data, y_dat, cl_dat, w_dat, var, con,
     n_y, no_of_out = len(y_dat), len(v_dict['y_name'])
     d_p, _, w_p, n_p = get_data_for_final_estimation(
         pred_data, v_dict, c_dict, ate=True,
-        need_count=c_dict['weight_as_sparse'])
+        need_count=c_dict['_weight_as_sparse'])
     if (d_p is not None) and (c_dict['atet_flag'] or c_dict['gatet_flag']):
         no_of_ates = no_of_treat + 1  # Compute ATEs, ATET, ATENT
     else:
@@ -85,7 +85,7 @@ def ate_est(weights, pred_data, y_dat, cl_dat, w_dat, var, con,
     w_ate = np.zeros((no_of_ates, no_of_treat, n_y))
     w_ate_export = np.zeros_like(w_ate)
     ind_d_val = np.arange(no_of_treat)
-    if c_dict['weight_as_sparse']:
+    if c_dict['_weight_as_sparse']:
         for i in range(n_p):
             w_add = np.zeros((no_of_treat, n_y))
             for t_ind, _ in enumerate(d_values):
@@ -266,6 +266,8 @@ def ate_est(weights, pred_data, y_dat, cl_dat, w_dat, var, con,
         ate = np.empty((no_of_out, no_of_ates,
                         round(no_of_treat * (no_of_treat - 1) / 2)))
     ate_se = np.empty_like(ate)
+    if balancing_test and c_dict['with_output'] and print_output:
+        ate_t = np.empty_like(ate)
     for o_idx, out_name in enumerate(v_dict['y_name']):
         if c_dict['with_output'] and print_output:
             print_str = f'Outcome variable: {out_name}'
@@ -294,6 +296,8 @@ def ate_est(weights, pred_data, y_dat, cl_dat, w_dat, var, con,
              ) = mcf_gp.effect_from_potential(
                 pot_y_ao, pot_y_var_ao, d_values, continuous=continuous)
             ate[o_idx, a_idx], ate_se[o_idx, a_idx] = est, stderr
+            if balancing_test and c_dict['with_output'] and print_output:
+                ate_t[o_idx, a_idx] = t_val
             if c_dict['with_output'] and print_output:
                 print_str = gp.print_effect(est, stderr, t_val, p_val,
                                             effect_list, continuous=continuous)
@@ -308,6 +312,17 @@ def ate_est(weights, pred_data, y_dat, cl_dat, w_dat, var, con,
                 if continuous and not balancing_test and a_idx == 0:
                     dose_response_figure(out_name, v_dict['d_name'][0], est,
                                          stderr, d_values[1:], c_dict)
+    if balancing_test and c_dict['with_output'] and print_output:
+        average_t = np.mean(ate_t)
+        print_str = '-' * 80 + '\n' + 'Balancing test\n'
+        print_str += 'Variables investigated:'
+        for name in v_dict['x_balance_name']:
+            print_str += f' {name}'
+        print_str += '\n' + '- ' * 40 + '\n' + 'Balancing test summary measure'
+        print_str += ' (average t-value of ATEs):'
+        print_str += f' {average_t:6.2f}' + '\n' + '-' * 80
+        print(print_str)
+        gp.print_f(c_dict['outfilesummary'], print_str)
     return w_ate_export, pot_y, pot_y_var, ate, ate_se, effect_list
 
 
