@@ -11,6 +11,9 @@ import copy
 import numpy as np
 import pandas as pd
 import matplotlib.pyplot as plt
+from sklearn.ensemble import RandomForestClassifier
+from sklearn.inspection import permutation_importance
+from sklearn.model_selection import train_test_split
 
 from mcf import mcf_data_functions as mcf_data
 from mcf import general_purpose as gp
@@ -76,15 +79,15 @@ def common_support(predict_file, tree_file, fill_y_file, fs_file, var_x_type,
             if len(var2-var1) > 0:
                 print('Variables in ', name2, 'not contained in ', name1,
                       *(var2-var1))
-            raise Exception(name1 + ' data and ' + name2 + ' data contain' +
-                            ' differnt variables. Programm stopped.')
+            raise ValueError(name1 + ' data and ' + name2 + ' data contain' +
+                             ' differnt variables. Programm stopped.')
 
     def mean_by_treatment(treat_pd, data_pd):
         treat_pd = treat_pd.squeeze()
         treat_vals = pd.unique(treat_pd)
         print('--------------- Mean by treatment status ------------------')
         if len(treat_vals) > 0:
-            mean = data_pd.groupby(treat_pd).mean()
+            mean = data_pd.groupby(treat_pd).mean(numeric_only=True)
             print(mean.transpose())
         else:
             print('All obs have same treatment:', treat_vals)
@@ -114,7 +117,7 @@ def common_support(predict_file, tree_file, fill_y_file, fs_file, var_x_type,
             print_str = ('-' * 80 + '\nData investigated and saved:'
                          + f'{out_file}' + '\n' + '-' * 80 + '\n'
                          + f'Observations deleted: {np.sum(obs_to_del_np):4}'
-                         + f' ({np.mean(obs_to_del_np)*100:.3f}%)'
+                         + f' ({np.mean(obs_to_del_np):.3%})'
                          + '\n' + '-' * 80)
             print(print_str)
             gp.print_f(c_dict['outfilesummary'], print_str)
@@ -182,21 +185,19 @@ def common_support(predict_file, tree_file, fill_y_file, fs_file, var_x_type,
                         mean_by_treatment(d_delete, x_delete)
                     else:
                         print('Only single observation deleted.')
-            assert np.mean(obs_to_del_np) <= c_dict['support_max_del_train'], (
-                'Less than {100-c_dict["support_max_del_train"]*100):3}% obs.'
-                + ' left after common support check of training data.'
-                + ' Programme terminated. Improve balance of input data before'
-                + ' forest building.')
+            if np.mean(obs_to_del_np) > c_dict['support_max_del_train']:
+                err_str = (
+                    'Less than {100-c_dict["support_max_del_train"]*100):3}% '
+                    + 'obs. left after common support check of training data.'
+                    + ' Programme terminated. Improve balance of input data'
+                    + ' before forest building.')
+                raise ValueError(err_str)
     if c_dict['d_type'] == 'continuous':
         d_name = v_dict['d_grid_nn_name']
         no_of_treat = len(c_dict['ct_grid_nn_val'])
     else:
         d_name, no_of_treat = v_dict['d_name'], c_dict['no_of_treat']
     x_name, x_type = gp.get_key_values_in_list(var_x_type)
-    # names_unordered = []  # Split ordered variables into dummies
-    # for j, val in enumerate(x_type):
-    #     if val > 0:
-    #         names_unordered.append(x_name[j])
     names_unordered = [x_name[j] for j, val in enumerate(x_type) if val > 0]
     fs_adjust, obs_fs = False, 0
     if c_dict['train_mcf']:
@@ -248,7 +249,6 @@ def common_support(predict_file, tree_file, fill_y_file, fs_file, var_x_type,
         d_all_in = pd.get_dummies(data_tr[d_name], columns=d_name)
         d_tr_np = d_all_in.to_numpy(copy=True)
         pred_tr_np = np.empty((np.shape(x_tr_np)[0], no_of_treat))
-    if c_dict['train_mcf']:
         x_pred_all = x_fy.copy()
         if c_dict['pred_mcf']:
             x_pred_all = pd.concat([x_pred_all, x_pr], axis=0)
@@ -274,36 +274,91 @@ def common_support(predict_file, tree_file, fill_y_file, fs_file, var_x_type,
             print('Computing random forest based common support')
     if c_dict['train_mcf']:
         cs_list = []
-        c_dict_new = mcf_data.m_n_grid(copy.deepcopy(c_dict),    # dict only
-                                       len(x_pred_all.columns))  # used here
-        for idx in range(no_of_treat):
-            return_forest = bool(c_dict['save_forest'])
-            ret_rf = gp_est.random_forest_scikit(
-                x_tr_np, d_tr_np[:, idx], x_pred_all_np, boot=c_dict['boot'],
-                n_min=c_dict_new['grid_n_min'],
-                no_features=c_dict_new['grid_m'], workers=workers_mp,
-                pred_p_flag=True, pred_t_flag=True, pred_oob_flag=True,
-                with_output=False, variable_importance=c_dict['verbose'],
-                x_name=x_name_all, var_im_with_output=c_dict['with_output'],
-                return_forest_object=return_forest)
-            pred_all_np[:, idx] = np.copy(ret_rf[0])
-            pred_tr_np[:, idx] = np.copy(ret_rf[1])
-            oob_best = np.copy(ret_rf[2])
+        if c_dict['support_cf']:
+            classif = RandomForestClassifier(
+                n_estimators=c_dict['boot'], max_features='sqrt',
+                bootstrap=True, oob_score=True, n_jobs=workers_mp,
+                random_state=42, verbose=False)
+            d_full_tr_np = data_tr[d_name].to_numpy().ravel()
+            x_train, x_test, d_train, d_test = train_test_split(
+                x_tr_np, d_full_tr_np, test_size=0.8, random_state=42)
+            classif.fit(x_train, d_train)
+            pred_all_np = classif.predict_proba(x_pred_all_np)
+            pred_tr_np = classif.predict_proba(x_tr_np)
+            pred_test = classif.predict_proba(x_test)
             if c_dict['save_forest']:
-                cs_list.append(ret_rf[6])
-            r2_obb(c_dict, idx, oob_best)
-            if no_of_treat == 2:
-                pred_all_np[:, idx+1] = 1 - pred_all_np[:, idx]
-                pred_tr_np[:, idx+1] = 1 - pred_tr_np[:, idx]
-                break
+                cs_list.append(classif)
+            if c_dict['with_output']:
+                print_str = 80 * '-' + '\n' + 'OOB score of classifier of'
+                print_str += 'propensity scores: '
+                print_str += f' {classif.oob_score_:10.7f}\n' + 80 * '-' + '\n'
+                vim = permutation_importance(classif, x_test, d_test,
+                                             n_jobs=workers_mp,
+                                             random_state=42)
+                ind_sort = np.flip(np.argsort(vim['importances_mean']))
+                print_str += 'Variable importance measures of propensity score'
+                print_str += ' (permutation, hold-out sample): Loss in % \n'
+                print_str += 40 * '- ' + '\n'
+                for idx in ind_sort:
+                    print_str += f'{x_tr.columns[idx]:30s}'
+                    relative_loss = (vim["importances_mean"][idx]
+                                     / classif.oob_score_ * 100)
+                    print_str += f'{relative_loss:10.4f}' + '\n'
+                print_str += 80 * '-'  + '\n'
+                print(print_str)
+                gp.print_f(c_dict['outfilesummary'], print_str)
+
+        else:                                       # dict only used here
+            c_dict_new = mcf_data.m_n_grid(copy.deepcopy(c_dict),
+                                           len(x_pred_all.columns))
+            if isinstance(c_dict_new['grid_n_min'], (tuple, list)):
+                n_min = (sum(c_dict_new['grid_n_min'])
+                         / len(c_dict_new['grid_n_min']) / 2)
+            else:
+                n_min = c_dict_new['grid_n_min'] / 2
+            for idx in range(no_of_treat):
+                ret_rf = gp_est.random_forest_scikit(
+                    x_tr_np, d_tr_np[:, idx], x_pred_all_np,
+                    boot=c_dict['boot'],
+                    n_min=n_min, no_features='sqrt', workers=workers_mp,
+                    pred_p_flag=True, pred_t_flag=True, pred_oob_flag=True,
+                    with_output=False, variable_importance=c_dict['verbose'],
+                    x_name=x_name_all,
+                    var_im_with_output=c_dict['with_output'],
+                    return_forest_object=c_dict['save_forest'])
+                pred_all_np[:, idx] = np.copy(ret_rf[0])
+                pred_tr_np[:, idx] = np.copy(ret_rf[1])
+                oob_best = np.copy(ret_rf[2])
+                if c_dict['save_forest']:
+                    cs_list.append(ret_rf[6])
+                r2_obb(c_dict, idx, oob_best)
+                if no_of_treat == 2:
+                    pred_all_np[:, idx+1] = 1 - pred_all_np[:, idx]
+                    pred_tr_np[:, idx+1] = 1 - pred_tr_np[:, idx]
+                    break
     else:
-        for idx in range(no_of_treat):
-            pred_all_np[:, idx] = cs_list[idx].predict(x_pred_all_np)
-            if no_of_treat == 2:
-                pred_all_np[:, idx+1] = 1 - pred_all_np[:, idx]
-                break
+        if c_dict['support_cf']:
+            d_train, d_test = train_test_split(
+                 d_tr_np, test_size=0.8, random_state=42)
+            pred_test = pred_tr_np[1]
+            pred_tr_np = pred_tr_np[0]
+            pred_all_np = cs_list[0].predict_proba(x_pred_all_np)
+        else:
+            for idx in range(no_of_treat):
+                pred_all_np[:, idx] = cs_list[idx].predict(x_pred_all_np)
+                if no_of_treat == 2:
+                    pred_all_np[:, idx+1] = 1 - pred_all_np[:, idx]
+                    break
+    if c_dict['support_cf']:
+        if c_dict['train_mcf']:
+            d_test_dummy = pd.get_dummies(d_test).to_numpy()
+        else:
+            d_test_dummy = d_test
+    else:
+        pred_test = d_test_dummy = None
     obs_to_del_all, obs_to_del_tr, upper_l, lower_l = off_support_and_plot(
-        pred_tr_np, pred_all_np, d_tr_np, c_dict)
+        pred_tr_np, pred_all_np, d_tr_np, c_dict, pred_out=pred_test,
+        d_out=d_test_dummy)
     # split obs_to_del_all into its parts
     obs_to_del_fs, obs_to_del_fy, obs_to_del_pr = False, False, False
     predict_file_new = None
@@ -337,6 +392,8 @@ def common_support(predict_file, tree_file, fill_y_file, fs_file, var_x_type,
             predict_file_new = predict_file
     else:
         predict_file_new = None
+    if c_dict['support_cf'] and c_dict['save_forest']:
+        pred_tr_np = (pred_tr_np.copy(), pred_test)
     return predict_file_new, cs_list, pred_tr_np, d_tr_np
 
 
@@ -356,7 +413,7 @@ def check_if_obs_needed(names_unordered, x_all_p, prime_values_dict):
                and not np.all(unique_vals_p == unique_vals_t)):
             print(name, 'Training values: ', unique_vals_t,
                   'Prediction values:', unique_vals_p)
-            raise Exception('Common support variable value error')
+            raise ValueError('Common support variable value error')
         add_vals_in_train = np.setdiff1d(unique_vals_t, unique_vals_p)
         if add_vals_in_train.size > 0:
             for i, val in enumerate(add_vals_in_train):
@@ -367,7 +424,8 @@ def check_if_obs_needed(names_unordered, x_all_p, prime_values_dict):
     return x_add_tmp
 
 
-def off_support_and_plot(pred_t, pred_p, d_t, c_dict):
+def off_support_and_plot(pred_t, pred_p, d_t, c_dict, pred_out=None,
+                         d_out=None):
     """
     Plot histogrammes and indicate which observations are off support.
 
@@ -377,6 +435,10 @@ def off_support_and_plot(pred_t, pred_p, d_t, c_dict):
     pred_p : N x no of treat Numpy array. Predictions of treat probs in pred.
     d_t: N x no of treat Numpy array. Treatment dummies.
     c_dict : Dict. Parameters.
+    pred_out : N x no of treat Numpy array. Predictions hold-out-sample.
+               Default is None.
+    d_out: N x no of treat Numpy array. Treatment dummies hold-out-sample.
+               Default is None.
 
     Returns
     -------
@@ -394,20 +456,24 @@ def off_support_and_plot(pred_t, pred_p, d_t, c_dict):
         d_values, no_of_treat = c_dict['d_values'], c_dict['no_of_treat']
     pred_t = pred_t / pred_t.sum(axis=1, keepdims=True)
     pred_p = pred_p / pred_p.sum(axis=1, keepdims=True)
+    if pred_out is not None:
+        pred_out = pred_out / pred_out.sum(axis=1, keepdims=True)
     n_p, n_t = np.shape(pred_p)[0], np.shape(pred_t)[0]
     q_s = c_dict['support_quantil']
     if c_dict['common_support'] == 1:
         upper_limit = np.empty((no_of_treat, no_of_treat))
         lower_limit = np.empty_like(upper_limit)
         for idx in range(no_of_treat):
-            if q_s == 1:
-                upper_limit[idx, :] = np.max(pred_t[d_t[:, idx] == 1], axis=0)
-                lower_limit[idx, :] = np.min(pred_t[d_t[:, idx] == 1], axis=0)
+            if pred_out is None:
+                probs = pred_t[d_t[:, idx] == 1]
             else:
-                upper_limit[idx, :] = np.quantile(pred_t[d_t[:, idx] == 1],
-                                                  q_s, axis=0)
-                lower_limit[idx, :] = np.quantile(pred_t[d_t[:, idx] == 1],
-                                                  1-q_s, axis=0)
+                probs = pred_out[d_out[:, idx] == 1]
+            if q_s == 1:
+                upper_limit[idx, :] = np.max(probs, axis=0)
+                lower_limit[idx, :] = np.min(probs, axis=0)
+            else:
+                upper_limit[idx, :] = np.quantile(probs, q_s, axis=0)
+                lower_limit[idx, :] = np.quantile(probs, 1 - q_s, axis=0)
         if c_dict['support_adjust_limits'] != 0:
             upper_limit *= 1 + c_dict['support_adjust_limits']
             lower_limit *= 1 - c_dict['support_adjust_limits']
@@ -457,7 +523,8 @@ def off_support_and_plot(pred_t, pred_p, d_t, c_dict):
         color_list = color_list[:len(d_values)]
         for idx_p, ival_p in enumerate(d_values):  # iterate treatment probs
             treat_prob = pred_t[:, idx_p]
-            titel = f'Probability of treatment {ival_p} in subsamples'
+            titel = (f'Probability of treatment {ival_p} in different '
+                     + 'subsamples')
             f_titel = f'common_support_pr_treat{ival_p}'
             file_name_jpeg = (c_dict['common_support_fig_pfad_jpeg']
                               + '/' + f_titel + '.jpeg')
@@ -467,9 +534,11 @@ def off_support_and_plot(pred_t, pred_p, d_t, c_dict):
                                 + '/' + f_titel + '_d.jpeg')
             file_name_pdf_d = (c_dict['common_support_fig_pfad_pdf']
                                + '/' + f_titel + '_d.pdf')
-            data_hist = []
-            for idx_sa, _ in enumerate(d_values):  # iterate treat.sample
-                data_hist.append(treat_prob[d_t[:, idx_sa] == 1])
+            # data_hist = []
+            # for idx_sa, _ in enumerate(d_values):  # iterate treat.sample
+            #     data_hist.append(treat_prob[d_t[:, idx_sa] == 1])
+            data_hist = [treat_prob[d_t[:, idx_sa] == 1]
+                         for idx_sa, _ in enumerate(d_values)]
             fig, axs = plt.subplots()
             fig_d, axs_d = plt.subplots()
             labels = ['Treat ' + str(d) for d in d_values]
