@@ -48,7 +48,7 @@ def plot_marginal(pred, pred_se, names_pred, x_name, x_values_in, x_type,
         x_values = x_values_in
     for idx, _ in enumerate(names_pred):
         if regrf:
-            titel = ('Marginal predictive plot ' + x_name + ' '
+            titel = ('Moderated predictive plot ' + x_name + ' '
                      + names_pred[idx])
         else:
             if minus_ate:
@@ -108,8 +108,9 @@ def plot_marginal(pred, pred_se, names_pred, x_name, x_values_in, x_type,
             print_str += '\n' + gp_est.print_se_info(c_dict['cluster_std'],
                                                      c_dict['se_boot_gate'])
             if minus_ate:
-                print_str += gp_est.print_minus_ate_info(c_dict['w_yes'],
-                                                         print_it=False)
+                if not c_dict['gates_minus_previous']:
+                    print_str += gp_est.print_minus_ate_info(c_dict['w_yes'],
+                                                             print_it=False)
             print(print_str)
             gp.print_f(c_dict['outfilesummary'], print_str)
         axs.set_ylabel(label_y)
@@ -274,12 +275,12 @@ def make_gate_figures_discr(
     if ate_se is None:
         gate_str = 'AMGATE-avg.(AMGATE)' if am_gate else 'GATE-ATE'
         gate_str_y = 'Effect - average'
-        ate_label = '_nolegend_'
         if am_gate:
             label_m = 'AMGATE-avg.(AMATE)'
         else:
             label_m = 'GATET-ATET' if gatet_yes else 'GATE-ATE'
         label_y = 'Effect - average'
+        ate_label = '_nolegend_'
     else:
         if am_gate:
             label_m = 'AMGATE'
@@ -691,7 +692,7 @@ def ref_file_marg_plot(plotx_name, c_dict, var_x_type, reference_values,
 def ref_vals_margplot(in_csv_file, var_x_type, var_x_values,
                       with_output=False, ref_values_needed=True,
                       no_eva_values=50):
-    """Compute reference values for marginal plots.
+    """Compute reference values for moderated plots.
 
     Parameters
     ----------
