@@ -9,6 +9,7 @@ Created on Thu Dec 8 15:48:57 2020.
 """
 from concurrent import futures
 from copy import deepcopy
+from dask.distributed import Client, as_completed
 
 import numpy as np
 import pandas as pd
@@ -65,19 +66,16 @@ def iate_est_mp(weights, data_file, y_dat, cl_dat, w_dat, v_dict, c_in_dict,
                   'nothing seems to happen, it may be worth to try do the ',
                   'estimation without sparse weight matrix. This needs more '
                   'memory, but could be substantially faster ',
-                  '(weight_as_sparse = False).')
+                  '(_weight_as_sparse = False).')
     if pot_y_prev is None:  # 1nd round of estimations
-        c_dict = c_in_dict
-        reg_round = True
+        c_dict, reg_round = c_in_dict, True
     else:
         c_dict = deepcopy(c_in_dict)
-        c_dict['iate_se_flag'] = False
-        c_dict['se_boot_iate'] = False
-        reg_round = False
+        c_dict['iate_se_flag'] = c_dict['se_boot_iate'] = reg_round = False
     if c_dict['with_output'] and c_dict['verbose'] and save_predictions:
         print('\nComputing IATEs 1/2 (potential outcomes)')
 
-    n_x = weights[0].shape[0] if c_dict['weight_as_sparse'] else len(weights)
+    n_x = weights[0].shape[0] if c_dict['_weight_as_sparse'] else len(weights)
     n_y, no_of_out = len(y_dat), len(v_dict['y_name'])
     if c_dict['d_type'] == 'continuous':
         no_of_treat, d_values = c_dict['ct_grid_w'], c_dict['ct_grid_w_val']
@@ -117,11 +115,11 @@ def iate_est_mp(weights, data_file, y_dat, cl_dat, w_dat, v_dict, c_in_dict,
             maxworkers = c_dict['no_parallel']
     if c_dict['with_output'] and c_dict['verbose']:
         print('Number of parallel processes: ', maxworkers)
-    if c_dict['weight_as_sparse']:
+    if c_dict['_weight_as_sparse']:
         iterator = len(weights)
     if maxworkers == 1:
         for idx in range(n_x):
-            if c_dict['weight_as_sparse']:
+            if c_dict['_weight_as_sparse']:
                 weights_idx = [weights[t_idx].getrow(idx) for
                                t_idx in range(iterator)]
             else:
@@ -137,7 +135,7 @@ def iate_est_mp(weights, data_file, y_dat, cl_dat, w_dat, v_dict, c_in_dict,
                 gp.share_completed(idx+1, n_x)
     else:
         if c_dict['obs_by_obs']:  # this is currently not used, too slow
-            if c_dict['mp_with_ray']:
+            if c_dict['_ray_or_dask'] == 'ray':
                 if c_dict['mem_object_store_3'] is None:
                     if not ray.is_initialized():
                         ray.init(num_cpus=maxworkers, include_dashboard=False)
@@ -149,7 +147,7 @@ def iate_est_mp(weights, data_file, y_dat, cl_dat, w_dat, v_dict, c_in_dict,
                     if c_dict['with_output'] and c_dict['verbose']:
                         print("Size of Ray Object Store: ", round(
                             c_dict['mem_object_store_3']/(1024*1024)), " MB")
-                if c_dict['weight_as_sparse']:
+                if c_dict['_weight_as_sparse']:
                     still_running = [ray_iate_func1_for_mp.remote(
                         idx, [weights[t_idx].getrow(idx) for t_idx in
                               range(iterator)], cl_dat, no_of_cluster, w_dat,
@@ -177,10 +175,34 @@ def iate_est_mp(weights, data_file, y_dat, cl_dat, w_dat, v_dict, c_in_dict,
                     del finished_res, finished
                 if c_dict['_mp_ray_shutdown']:
                     ray.shutdown()
+            elif c_dict['_ray_or_dask'] == 'dask':
+                with Client(n_workers=maxworkers) as clt:
+                    if c_dict['_weight_as_sparse']:
+                        ret_fut = [clt.submit(
+                            iate_func1_for_mp, idx,
+                            [weights[t_idx].getrow(idx) for t_idx in
+                             range(iterator)], cl_dat, no_of_cluster, w_dat,
+                            w_ate, y_dat, no_of_out, n_y, c_dict)
+                            for idx in range(n_x)]
+                        warn_text(c_dict)
+                    else:
+                        ret_fut = [clt.submit(
+                            iate_func1_for_mp, idx, weights[idx], cl_dat,
+                            no_of_cluster, w_dat, w_ate, y_dat, no_of_out, n_y,
+                            c_dict) for idx in range(n_x)]
+                    jdx = 0
+                    for _, res in as_completed(ret_fut, with_results=True):
+                        jdx += 1
+                        (pot_y, pot_y_var, pot_y_m_ate, pot_y_m_ate_var,
+                         l1_to_9, share_censored) = assign_ret_all_i(
+                             pot_y, pot_y_var, pot_y_m_ate, pot_y_m_ate_var,
+                             l1_to_9, share_censored, res, n_x)
+                        if c_dict['with_output'] and c_dict['verbose']:
+                            gp.share_completed(jdx+1, n_x)
             else:
                 with futures.ProcessPoolExecutor(max_workers=maxworkers
                                                  ) as fpp:
-                    if c_dict['weight_as_sparse']:
+                    if c_dict['_weight_as_sparse']:
                         ret_fut = {fpp.submit(
                             iate_func1_for_mp, idx,
                             [weights[t_idx].getrow(idx) for t_idx in
@@ -212,7 +234,7 @@ def iate_est_mp(weights, data_file, y_dat, cl_dat, w_dat, v_dict, c_in_dict,
                       f'{n_x / no_of_splits:5.2f}.',
                       ' Number of splits: ', no_of_splits)
             obs_idx_list = np.array_split(np.arange(n_x), no_of_splits)
-            if c_dict['mp_with_ray']:
+            if c_dict['_ray_or_dask'] == 'ray':
                 if c_dict['mem_object_store_3'] is None:
                     if not ray.is_initialized():
                         ray.init(num_cpus=maxworkers, include_dashboard=False)
@@ -224,7 +246,7 @@ def iate_est_mp(weights, data_file, y_dat, cl_dat, w_dat, v_dict, c_in_dict,
                     if c_dict['with_output'] and c_dict['verbose']:
                         print("Size of Ray Object Store: ", round(
                             c_dict['mem_object_store_3']/(1024*1024)), " MB")
-                if c_dict['weight_as_sparse']:
+                if c_dict['_weight_as_sparse']:
                     still_running = [ray_iate_func1_for_mp_many_obs.remote(
                         idx, [weights[t_idx][idx, :] for t_idx in
                               range(iterator)], cl_dat, no_of_cluster,
@@ -253,10 +275,35 @@ def iate_est_mp(weights, data_file, y_dat, cl_dat, w_dat, v_dict, c_in_dict,
                     del finished_res, finished
                 if c_dict['_mp_ray_shutdown']:
                     ray.shutdown()
+            elif c_dict['_ray_or_dask'] == 'dask':
+                with Client(n_workers=maxworkers) as clt:
+                    if c_dict['_weight_as_sparse']:
+                        ret_fut = [clt.submit(
+                            iate_func1_for_mp_many_obs, idx,
+                            [weights[t_idx][idx, :] for t_idx in
+                             range(iterator)], cl_dat, no_of_cluster, w_dat,
+                            w_ate, y_dat, no_of_out, n_y, c_dict)
+                                for idx in obs_idx_list]
+                    else:
+                        ret_fut = [clt.submit(
+                            iate_func1_for_mp_many_obs, idx,
+                            [weights[idxx] for idxx in idx], cl_dat,
+                            no_of_cluster, w_dat, w_ate, y_dat, no_of_out,
+                            n_y, c_dict) for idx in obs_idx_list]
+                    jdx = 0
+                    for _, res_l in as_completed(ret_fut, with_results=True):
+                        jdx += 1
+                        for res in res_l:
+                            (pot_y, pot_y_var, pot_y_m_ate, pot_y_m_ate_var,
+                             l1_to_9, share_censored) = assign_ret_all_i(
+                             pot_y, pot_y_var, pot_y_m_ate, pot_y_m_ate_var,
+                             l1_to_9, share_censored, res, n_x)
+                        if c_dict['with_output'] and c_dict['verbose']:
+                            gp.share_completed(jdx+1, no_of_splits)
             else:
                 with futures.ProcessPoolExecutor(max_workers=maxworkers
                                                  ) as fpp:
-                    if c_dict['weight_as_sparse']:
+                    if c_dict['_weight_as_sparse']:
                         ret_fut = {fpp.submit(
                             iate_func1_for_mp_many_obs, idx,
                             [weights[t_idx][idx, :] for t_idx in
@@ -315,7 +362,6 @@ def iate_est_mp(weights, data_file, y_dat, cl_dat, w_dat, v_dict, c_in_dict,
         iate_se, iate_p = np.empty_like(iate), np.empty_like(iate)
     else:
         iate_se = iate_p = None
-    # obs x outcome x effects x type_of_effect
     if c_dict['no_parallel'] < 1.5:
         maxworkers = 1
     else:
@@ -346,7 +392,7 @@ def iate_est_mp(weights, data_file, y_dat, cl_dat, w_dat, v_dict, c_in_dict,
             if idx == n_x-1:
                 effect_list = ret_all_idx[4]
     else:
-        if c_dict['mp_with_ray']:
+        if c_dict['_ray_or_dask'] == 'ray':
             if c_dict['mem_object_store_3'] is None:
                 if not ray.is_initialized():
                     ray.init(num_cpus=maxworkers, include_dashboard=False)
@@ -387,6 +433,30 @@ def iate_est_mp(weights, data_file, y_dat, cl_dat, w_dat, v_dict, c_in_dict,
                 del finished_res, finished
             if c_dict['_mp_ray_shutdown']:
                 ray.shutdown()
+        elif c_dict['_ray_or_dask'] == 'dask':
+            with Client(n_workers=maxworkers) as clt:
+                if c_dict['iate_se_flag']:
+                    ret_fut = [clt.submit(
+                        iate_func2_for_mp, idx, no_of_out, pot_y[idx],
+                        pot_y_var[idx], pot_y_m_ate[idx], pot_y_m_ate_var[idx],
+                        c_dict, d_values_dr, no_of_treat_dr)
+                        for idx in range(n_x)]
+                else:
+                    ret_fut = [clt.submit(
+                        iate_func2_for_mp, idx, no_of_out, pot_y[idx],
+                        None, None, None, c_dict, d_values_dr, no_of_treat_dr)
+                        for idx in range(n_x)]
+                jdx = 0
+                for _, res in as_completed(ret_fut, with_results=True):
+                    jdx += 1
+                    iix, iate[iix, :, :, :] = res[0], res[1]
+                    if c_dict['iate_se_flag']:
+                        iate_se[iix, :, :, :] = res[2]
+                        iate_p[iix, :, :, :] = res[3]
+                    if jdx == n_x-1:
+                        effect_list = res[4]
+                    if c_dict['with_output'] and c_dict['verbose']:
+                        gp.share_completed(jdx+1, n_x)
         else:
             with futures.ProcessPoolExecutor(max_workers=maxworkers) as fpp:
                 ret_fut = {fpp.submit(
@@ -425,8 +495,7 @@ def iate_est_mp(weights, data_file, y_dat, cl_dat, w_dat, v_dict, c_in_dict,
     jdx = j2dx = jdx_unlc = 0
     name_pot, name_eff, name_eff0 = [], [], []
     if c_dict['l_centering_uncenter']:
-        name_pot_unlc = []
-        pot_y_unlc_np = np.empty((n_x, no_of_treat_dr))
+        name_pot_unlc, pot_y_unlc_np = [], np.empty((n_x, no_of_treat_dr))
         if isinstance(v_dict['y_tree_name'], list):
             y_tree_name = v_dict['y_tree_name'][0]
         else:
@@ -479,8 +548,8 @@ def iate_est_mp(weights, data_file, y_dat, cl_dat, w_dat, v_dict, c_in_dict,
         name_pot_y_se = name_iate_se = name_iate_mate = None
         name_iate_mate_se = name_iate_se0 = name_iate_mate0 = None
         name_iate_mate_se0 = None
-    if (c_dict['with_output'] and save_predictions
-        ) or c_dict['_return_iate_sp']:
+    if (c_dict['with_output'] and save_predictions) or c_dict[
+            '_return_iate_sp']:
         pot_y_df = pd.DataFrame(data=pot_y_np, columns=name_pot_y)
         iate_df = pd.DataFrame(data=iate_np, columns=name_iate)
         if c_dict['iate_se_flag']:
@@ -501,18 +570,23 @@ def iate_est_mp(weights, data_file, y_dat, cl_dat, w_dat, v_dict, c_in_dict,
             df_list = [data_df, pot_y_df, iate_df]
         if c_dict['l_centering_uncenter']:
             x_pred_np = find_x_to_uncenter(data_df, var_x_type)
-            y_x_lc = lc_forest.predict(x_pred_np)
+            try:
+                y_x_lc = lc_forest.predict(x_pred_np)
+            except RuntimeError:
+                y_x_lc = np.empty(len(x_pred_np)) * np.nan
+                if c_dict['with_output']:
+                    print('Uncentering not successful.')
             pot_y_unlc_np += np.reshape(y_x_lc, (-1, 1))
             pot_y_unlc_df = pd.DataFrame(data=pot_y_unlc_np,
                                          columns=name_pot_y_unlc)
             df_list.append(pot_y_unlc_df)
-        data_file_new = pd.concat(df_list, axis=1)
+        data_pred_new = pd.concat(df_list, axis=1)
         if ((c_dict['with_output'] and save_predictions)
-            or (save_predictions and reg_round and c_dict['effiate_flag'])
-            or (save_predictions and c_dict['effiate_flag']
+            or (save_predictions and reg_round and c_dict['iate_eff_flag'])
+            or (save_predictions and c_dict['iate_eff_flag']
                 and c_dict['_return_iate_sp'])):
             gp.delete_file_if_exists(c_dict['pred_sample_with_pred'])
-            data_file_new.to_csv(c_dict['pred_sample_with_pred'], index=False)
+            data_pred_new.to_csv(c_dict['pred_sample_with_pred'], index=False)
         if c_dict['with_output']:
             gp.print_descriptive_stats_file(
                 c_dict['pred_sample_with_pred'], 'all',
@@ -520,31 +594,26 @@ def iate_est_mp(weights, data_file, y_dat, cl_dat, w_dat, v_dict, c_in_dict,
     names_pot_iate = {
         'names_pot_y': name_pot_y, 'names_pot_y_se': name_pot_y_se,
         'names_iate': name_iate, 'names_iate_se': name_iate_se,
-        'names_iate_mate': name_iate_mate,
-        'names_iate_mate_se': name_iate_mate_se,
-        'names_pot_y_uncenter': name_pot_y_unlc}
+        'names_iate_mate': name_iate_mate, 'names_iate_mate_se':
+            name_iate_mate_se, 'names_pot_y_uncenter': name_pot_y_unlc}
     names_pot_iate0 = {
         'names_pot_y': name_pot_y, 'names_pot_y_se': name_pot_y_se,
         'names_iate': name_iate0, 'names_iate_se': name_iate_se0,
-        'names_iate_mate': name_iate_mate0,
-        'names_iate_mate_se': name_iate_mate_se0,
-        'names_pot_y_uncenter': name_pot_y_unlc}
+        'names_iate_mate': name_iate_mate0, 'names_iate_mate_se':
+            name_iate_mate_se0, 'names_pot_y_uncenter': name_pot_y_unlc}
     if not c_dict['_return_iate_sp']:
-        data_file_new = None
+        data_pred_new = None
     return (c_dict['pred_sample_with_pred'], pot_y, pot_y_var, iate, iate_se,
-            (names_pot_iate, names_pot_iate0), data_file_new)
+            (names_pot_iate, names_pot_iate0), data_pred_new)
 
 
 def find_x_to_uncenter(data_df, var_x_type):
-    "Find correct x to uncenter potential outcomes."
+    """Find correct x to uncenter potential outcomes."""
     x_names = var_x_type.keys()
     x_df = data_df[x_names]
     # This part must be identical to the same part in local centering!
     # To Do: Use same function in both.
-    names_unordered = []
-    for x_name in x_names:
-        if var_x_type[x_name] > 0:
-            names_unordered.append(x_name)
+    names_unordered = [x_name for x_name in x_names if var_x_type[x_name] > 0]
     if names_unordered:  # List is not empty
         x_dummies = pd.get_dummies(x_df, columns=names_unordered)
         x_df = pd.concat([x_df[names_unordered], x_dummies], axis=1)
@@ -646,10 +715,10 @@ def iate_func1_for_mp_many_obs(idx_list, weights_list, cl_dat, no_of_cluster,
                                w_dat, w_ate, y_dat, no_of_out, n_y, c_dict):
     """Compute IATE for several obs in one loop (MP)."""
     ret_all = []
-    if c_dict['weight_as_sparse']:
+    if c_dict['_weight_as_sparse']:
         iterator = len(weights_list)
     for i, idx_org in enumerate(idx_list):
-        if c_dict['weight_as_sparse']:
+        if c_dict['_weight_as_sparse']:
             weights_i = [weights_list[t_idx].getrow(i)
                          for t_idx in range(iterator)]
         else:
@@ -696,27 +765,25 @@ def iate_func1_for_mp(idx, weights_i, cl_dat, no_of_cluster, w_dat, w_ate,
     pot_y_m_ate_var_i: Numpy array.
     l1_to_9: Tuple of lists.
     """
-    def get_walli(w_index, n_y, w_i):
+    def get_walli(w_index, n_y, w_i, w_i_unc):
         w_all_i = np.zeros(n_y)
         w_all_i[w_index] = w_i
         w_all_i_unc = np.zeros_like(w_all_i)
         w_all_i_unc[w_index] = w_i_unc
         return w_all_i, w_all_i_unc
 
-    if c_dict['with_output'] and (idx == 0) and not c_dict[
-            'mp_with_ray'] and c_dict['verbose']:
+    if (c_dict['with_output'] and (idx == 0)
+            and c_dict['_ray_or_dask'] != 'ray' and c_dict['verbose']):
         print('Starting to compute IATE - procedure 1', flush=True)
     if c_dict['d_type'] == 'continuous':
-        continuous = True
+        continuous, d_values_dr = True, c_dict['ct_d_values_dr_np']
         no_of_treat = c_dict['ct_grid_w']
         i_w01 = c_dict['ct_w_to_dr_int_w01']
         i_w10 = c_dict['ct_w_to_dr_int_w10']
         index_full = c_dict['ct_w_to_dr_index_full']
-        d_values_dr = c_dict['ct_d_values_dr_np']
         no_of_treat_dr = len(d_values_dr)
     else:
-        continuous = False
-        d_values_dr = c_dict['d_values']
+        continuous, d_values_dr = False, c_dict['d_values']
         no_of_treat = no_of_treat_dr = c_dict['no_of_treat']
     pot_y_i = np.empty((no_of_treat_dr, no_of_out))
     share_i = np.zeros(no_of_treat_dr)
@@ -734,36 +801,30 @@ def iate_func1_for_mp(idx, weights_i, cl_dat, no_of_cluster, w_dat, w_ate,
         w_add_unc = np.zeros((no_of_treat_dr, n_y))
     for t_idx in range(no_of_treat):
         extra_weight_p1 = continuous and t_idx < no_of_treat-1
-        if c_dict['weight_as_sparse']:
+        if c_dict['_weight_as_sparse']:
             w_index = weights_i[t_idx].indices
             w_i_t = weights_i[t_idx].data
-            if extra_weight_p1:
-                w_index_p1 = weights_i[t_idx+1].indices
-                w_index_both = np.unique(np.concatenate((w_index, w_index_p1)))
-                w_i = np.zeros(n_y)
-                w_i[w_index] = w_i_t
-                w_i_p1 = np.zeros_like(w_i)
-                w_i_p1[w_index_p1] = weights_i[t_idx+1].data
-                w_i = w_i[w_index_both]
-                w_i_p1 = w_i_p1[w_index_both]
-            else:
-                w_index_both = w_index
-                w_i = w_i_t
         else:
             w_index = weights_i[t_idx][0]    # Indices of non-zero weights
             w_i_t = weights_i[t_idx][1].copy()
-            if extra_weight_p1:
-                w_index_p1 = weights_i[t_idx+1][0]
-                w_index_both = np.unique(np.concatenate((w_index, w_index_p1)))
-                w_i = np.zeros(n_y)
-                w_i[w_index] = w_i_t
-                w_i_p1 = np.zeros_like(w_i)
-                w_i_p1[w_index_p1] = weights_i[t_idx+1][1].copy()
-                w_i = w_i[w_index_both]
-                w_i_p1 = w_i_p1[w_index_both]
+        if extra_weight_p1:
+            if c_dict['_weight_as_sparse']:
+                w_index_p1 = weights_i[t_idx+1].indices
             else:
-                w_index_both = w_index
-                w_i = w_i_t
+                w_index_p1 = weights_i[t_idx+1][0]
+            w_index_both = np.unique(np.concatenate((w_index, w_index_p1)))
+            w_i = np.zeros(n_y)
+            w_i[w_index] = w_i_t
+            w_i_p1 = np.zeros_like(w_i)
+            if c_dict['_weight_as_sparse']:
+                w_i_p1[w_index_p1] = weights_i[t_idx+1].data
+            else:
+                w_i_p1[w_index_p1] = weights_i[t_idx+1][1].copy()
+            w_i = w_i[w_index_both]
+            w_i_p1 = w_i_p1[w_index_both]
+        else:
+            w_index_both = w_index
+            w_i = w_i_t
         if c_dict['w_yes']:
             w_t = w_dat[w_index].reshape(-1)
             w_i = w_i * w_t
@@ -784,17 +845,18 @@ def iate_func1_for_mp(idx, weights_i, cl_dat, no_of_cluster, w_dat, w_ate,
         if extra_weight_p1:
             w_i_unc_p1 = np.copy(w_i_p1)
         if cluster_std:
-            w_all_i, w_all_i_unc = get_walli(w_index, n_y, w_i)
+            w_all_i, w_all_i_unc = get_walli(w_index, n_y, w_i, w_i_unc)
             cl_i = cl_dat[w_index]
             if extra_weight_p1:
-                w_all_i_p1, w_all_i_unc_p1 = get_walli(w_index_p1, n_y, w_i_p1)
+                w_all_i_p1, w_all_i_unc_p1 = get_walli(w_index_p1, n_y, w_i_p1,
+                                                       w_i_unc_p1)
                 cl_i_both = cl_dat[w_index_both]
         else:
             cl_i = cl_i_both = None
         for o_idx in range(no_of_out):
             if continuous:
+                y_dat_cont = y_dat[w_index_both, o_idx]
                 for i, (w01, w10) in enumerate(zip(i_w01, i_w10)):
-                    y_dat_cont = y_dat[w_index_both, o_idx]
                     if extra_weight_p1:
                         w_i_cont = w10 * w_i + w01 * w_i_p1
                         w_i_unc_cont = w10 * w_i_unc + w01 * w_i_unc_p1
