@@ -47,9 +47,9 @@ def sub_size(n_train, subsample_share_mult, max_share):
         subsample_share_mult = -1
     if subsample_share_mult <= 0:
         subsample_share_mult = 1
-    subsam_share = 2 * ((n_train / 2)**0.85) / (n_train / 2)
+    subsam_share = 4 * ((n_train / 2)**0.85) / n_train
     subsam_share = min(subsam_share, 0.67)
-    subsam_share = subsam_share * subsample_share_mult
+    subsam_share *= subsample_share_mult
     subsam_share = min(subsam_share, max_share)
     subsam_share = max(subsam_share, 1e-4)
     return subsam_share
@@ -89,10 +89,10 @@ def controls_into_dic(
         min_dummy_obs, mce_vart, p_diff_penalty, boot, n_min_max,
         support_min_p, weighted, support_check, support_quantil,
         subsample_factor_forest, subsample_factor_eval,
-        m_min_share, m_grid, m_random_poisson, alpha_reg_min,
+        m_share_min, m_grid, m_random_poisson, alpha_reg_min,
         alpha_reg_max, alpha_reg_grid,
         random_thresholds, knn_min_k, share_forest_sample, descriptive_stats,
-        m_max_share, max_cats_z_vars, variable_importance_oob,
+        m_share_max, max_cats_z_vars, variable_importance_oob,
         balancing_test, choice_based_sampling, knn_const, choice_based_weights,
         nw_kern_flag, post_kmeans_max_tries, cond_var_flag, knn_flag, nw_bandw,
         panel_data, max_cats_cont_vars, cluster_std, fs_yes,
@@ -106,8 +106,8 @@ def controls_into_dic(
         post_random_forest_vi, gmate_no_evaluation_points,
         gmate_sample_share, no_filled_plot, smooth_gates,
         smooth_gates_bandwidth, smooth_gates_no_evaluation_points, show_plots,
-        weight_as_sparse, mp_type_weights, mp_weights_tree_batch,
-        boot_by_boot, obs_by_obs, max_elements_per_split, mp_with_ray,
+        _weight_as_sparse, mp_type_weights, mp_weights_tree_batch,
+        boot_by_boot, obs_by_obs, max_elements_per_split,
         mp_ray_objstore_multiplier, verbose, no_ray_in_forest_building,
         predict_mcf, train_mcf, forest_files, match_nn_prog_score,
         se_boot_ate, se_boot_gate, se_boot_iate, support_max_del_train,
@@ -117,7 +117,9 @@ def controls_into_dic(
         reduce_largest_group_train, reduce_largest_group_train_share,
         iate_flag, iate_se_flag, l_centering_uncenter, d_type, ct_grid_nn,
         ct_grid_w, ct_grid_dr, support_adjust_limits, l_centering_replication,
-        effiate_flag, _return_iate_sp):
+        iate_eff_flag, _return_iate_sp, iate_cv_flag, iate_cv_folds,
+        n_min_treat, gates_minus_previous, _ray_or_dask,
+        _weight_as_sparse_splits, support_cf):
     """Build dictionary with parameters.
 
     Parameters
@@ -139,16 +141,17 @@ def controls_into_dic(
         'common_support': support_check, 'support_quantil': support_quantil,
         'support_adjust_limits': support_adjust_limits,
         'support_max_del_train': support_max_del_train,
+        'support_cf': support_cf,
         'w_yes': weighted, 'knn_min_k': knn_min_k, 'n_min_max': n_min_max,
         'nw_bandw': nw_bandw, 'nw_kern': nw_kern_flag,
         'subsample_factor_forest': subsample_factor_forest,
         'subsample_factor_eval': subsample_factor_eval,
-        'm_min_share': m_min_share, 'm_grid': m_grid,
+        'm_share_min': m_share_min, 'm_grid': m_grid,
         'm_random_poisson': m_random_poisson, 'alpha_reg_min': alpha_reg_min,
         'alpha_reg_max': alpha_reg_max, 'alpha_reg_grid': alpha_reg_grid,
         'random_thresholds': random_thresholds, 'knn_const': knn_const,
         'share_forest_sample': share_forest_sample, 'knn': knn_flag,
-        'desc_stat': descriptive_stats, 'm_max_share': m_max_share,
+        'desc_stat': descriptive_stats, 'm_share_max': m_share_max,
         'var_import_oob': variable_importance_oob, 'cond_var': cond_var_flag,
         'balancing_test_w': balancing_test, 'max_cats_z_vars': max_cats_z_vars,
         'choice_based_yes': choice_based_sampling, 'panel_data': panel_data,
@@ -180,12 +183,12 @@ def controls_into_dic(
         'no_filled_plot': no_filled_plot, 'smooth_gates': smooth_gates,
         'sgates_bandwidth': smooth_gates_bandwidth,
         'sgates_no_evaluation_points': smooth_gates_no_evaluation_points,
-        'show_plots': show_plots, 'weight_as_sparse': weight_as_sparse,
+        'show_plots': show_plots, '_weight_as_sparse': _weight_as_sparse,
         'mp_type_weights': mp_type_weights, 'mp_weights_tree_batch':
         mp_weights_tree_batch, 'boot_by_boot': boot_by_boot, 'obs_by_obs':
         obs_by_obs, 'max_elements_per_split': max_elements_per_split,
-        'mp_with_ray': mp_with_ray, 'mp_ray_objstore_multiplier':
-        mp_ray_objstore_multiplier, 'verbose': verbose,
+        'mp_ray_objstore_multiplier': mp_ray_objstore_multiplier,
+        'verbose': verbose,
         'no_ray_in_forest_building': no_ray_in_forest_building,
         'pred_mcf': predict_mcf, 'train_mcf': train_mcf,
         'save_forest_files': forest_files,
@@ -201,10 +204,15 @@ def controls_into_dic(
         'reduce_largest_group_train': reduce_largest_group_train,
         'reduce_largest_group_train_share': reduce_largest_group_train_share,
         'iate_flag': iate_flag, 'iate_se_flag': iate_se_flag,
-        'effiate_flag': effiate_flag,
+        'iate_eff_flag': iate_eff_flag,
         'd_type': d_type, 'ct_grid_nn': ct_grid_nn, 'ct_grid_w': ct_grid_w,
         'ct_grid_dr': ct_grid_dr,
-        '_return_iate_sp': _return_iate_sp
+        '_return_iate_sp': _return_iate_sp,
+        'iate_cv_flag': iate_cv_flag,  'iate_cv_folds': iate_cv_folds,
+        'n_min_treat': n_min_treat,
+        'gates_minus_previous': gates_minus_previous,
+        '_ray_or_dask': _ray_or_dask,
+        '_weight_as_sparse_splits': _weight_as_sparse_splits
             }
     return controls_dict
 
