@@ -5,7 +5,6 @@ Contains the functions needed for the running all parts of the programme
 -*- coding: utf-8 -*-
 """
 import copy
-import math
 from pathlib import Path
 import os
 
@@ -49,22 +48,22 @@ def get_controls(c_dict, v_dict):
             try:
                 os.mkdir(out_temp)
             except OSError as oserr:
-                raise Exception(
+                raise OSError(
                     f'Creation of the directory {out_temp} failed') from oserr
-            else:
-                if not ((c_dict['with_output'] is False)
-                        or (c_dict['verbose'] is False)):
-                    print(f'Successfully created the directory {out_temp}')
-                if out_temp != c_dict['outpfad']:
-                    c_dict['outpfad'] = out_temp
-                break
+            if not ((c_dict['with_output'] is False)
+                    or (c_dict['verbose'] is False)):
+                print(f'Successfully created the directory {out_temp}')
+            if out_temp != c_dict['outpfad']:
+                c_dict['outpfad'] = out_temp
+            break
     if (c_dict['indata'] is None) and c_dict['train_mcf']:
-        raise Exception('Filename of indata must be specified')
+        raise FileNotFoundError('Filename of indata must be specified')
     if (c_dict['preddata'] is None) and c_dict['train_mcf']:
         c_dict['preddata'] = c_dict['indata']
     if c_dict['outfiletext'] is None:
         c_dict['outfiletext'] = (c_dict['indata'] if c_dict['train_mcf'] else
                                  c_dict['preddata'])
+
     cnew_dict = copy.deepcopy(c_dict)
 
     temppfad = c_dict['outpfad'] + '/_tempmcf_'
@@ -82,12 +81,11 @@ def get_controls(c_dict, v_dict):
         try:
             os.mkdir(temppfad)
         except OSError as oserr:
-            raise Exception(f'Creation of the directory {temppfad} failed'
-                            ) from oserr
-        else:
-            if not ((c_dict['with_output'] is False)
-                    or (c_dict['verbose'] is False)):
-                print(f'Successfully created the directory {temppfad}')
+            raise OSError(f'Creation of the directory {temppfad} failed'
+                          ) from oserr
+        if not ((c_dict['with_output'] is False)
+                or (c_dict['verbose'] is False)):
+            print(f'Successfully created the directory {temppfad}')
     make_dir = c_dict['pred_mcf'] and c_dict['with_output']
     cnew_dict = mcf_ia.get_fig_path(cnew_dict, 'cs_ate_iate', make_dir)
     cnew_dict = mcf_ia.get_fig_path(cnew_dict, 'gate', make_dir)
@@ -149,11 +147,9 @@ def get_controls(c_dict, v_dict):
     if c_dict['output_type'] is None:
         c_dict['output_type'] = 2
     if c_dict['output_type'] == 0:
-        print_to_file = False
-        print_to_terminal = True
+        print_to_file, print_to_terminal = False, True
     elif c_dict['output_type'] == 1:
-        print_to_file = True
-        print_to_terminal = False
+        print_to_file, print_to_terminal = True, False
     else:
         print_to_file = print_to_terminal = True
     cnew_dict['verbose'] = not c_dict['verbose'] is False
@@ -167,7 +163,7 @@ def get_controls(c_dict, v_dict):
     elif c_dict['d_type'] == 'continuous':
         cnew_dict['d_type'] = 'continuous'
     else:
-        raise Exception(f'{c_dict["d_type"]} is wrong treatment type.')
+        raise ValueError(f'{c_dict["d_type"]} is wrong treatment type.')
 
     cnew_dict['screen_covariates'] = not c_dict['screen_covariates'] is False
     cnew_dict['check_perfectcorr'] = not c_dict['check_perfectcorr'] is False
@@ -176,30 +172,30 @@ def get_controls(c_dict, v_dict):
     cnew_dict['min_dummy_obs'] = (10 if c_dict['min_dummy_obs'] < 1
                                   else round(c_dict['min_dummy_obs']))
     cnew_dict['clean_data_flag'] = not c_dict['clean_data_flag'] is False
-    if c_dict['alpha_reg_min'] is None:
-        c_dict['alpha_reg_min'] = -1
-    if (c_dict['alpha_reg_min'] < 0) or (c_dict['alpha_reg_min'] >= 0.4):
-        cnew_dict['alpha_reg_min'] = 0.1
-        if c_dict['alpha_reg_min'] >= 0.4:
+    if (c_dict['alpha_reg_min'] is None or c_dict['alpha_reg_min'] < 0
+            or c_dict['alpha_reg_min'] >= 0.4):
+        cnew_dict['alpha_reg_min'] = 0.05
+        if (c_dict['alpha_reg_min'] is not None
+                and c_dict['alpha_reg_min'] >= 0.4):
             print('Values of 0.4 and larger do not make sense for alpha',
                   'regularisation. alpha regularisation parameter set to ',
                   f'default value. {cnew_dict["alpha_reg_min"]:8.3f}')
     else:
         cnew_dict['alpha_reg_min'] = c_dict['alpha_reg_min']
-    if c_dict['alpha_reg_max'] is None:
-        c_dict['alpha_reg_max'] = -1
-    if (c_dict['alpha_reg_max'] < 0) or (c_dict['alpha_reg_max'] >= 0.4):
-        cnew_dict['alpha_reg_max'] = 0.1
-        if c_dict['alpha_reg_max'] >= 0.4:
+    if (c_dict['alpha_reg_max'] is None or c_dict['alpha_reg_max'] < 0
+            or c_dict['alpha_reg_max'] >= 0.4):
+        cnew_dict['alpha_reg_max'] = 0.15
+        if (c_dict['alpha_reg_max'] is not None
+                and c_dict['alpha_reg_max'] >= 0.4):
             print('Values of 0.4 and larger do not make sense for alpha',
                   'regularisation. alpha regularisation parameter set to ',
                   f'default value. {cnew_dict["alpha_reg_max"]:8.3f}')
     else:
         cnew_dict['alpha_reg_max'] = c_dict['alpha_reg_max']
-    cnew_dict['alpha_reg_grid'] = (1 if c_dict['alpha_reg_grid'] is None
-                                   else round(c_dict['alpha_reg_grid']))
-    if cnew_dict['alpha_reg_grid'] < 1:
+    if c_dict['alpha_reg_grid'] is None or c_dict['alpha_reg_grid'] < 1:
         cnew_dict['alpha_reg_grid'] = 1
+    else:
+        cnew_dict['alpha_reg_grid'] = round(c_dict['alpha_reg_grid'])
     if cnew_dict['alpha_reg_min'] >= cnew_dict['alpha_reg_max']:
         cnew_dict['alpha_reg_grid'] = 1
         cnew_dict['alpha_reg_max'] = cnew_dict['alpha_reg_min']
@@ -228,9 +224,10 @@ def get_controls(c_dict, v_dict):
         panel = True
     else:
         cnew_dict['panel_data'] = panel = cnew_dict['panel_in_rf'] = False
-    data_train = (pd.read_csv(cnew_dict['indata'], header=0)
-                  if c_dict['train_mcf']
-                  else pd.read_csv(save_forest_file_csv, header=0))
+    if c_dict['train_mcf']:
+        data_train = pd.read_csv(cnew_dict['indata'], header=0)
+    else:
+        data_train = pd.read_csv(save_forest_file_csv, header=0)
     v_dict['d_name'] = mcf_ia.to_list_if_needed(v_dict['d_name'])
     v_dict['d_name'][0] = gp.adjust_var_name(v_dict['d_name'][0],
                                              data_train.columns.tolist())
@@ -272,84 +269,110 @@ def get_controls(c_dict, v_dict):
             print('Counting treatments does not work. ' +
                   f'n_d_sum: {n_d.sum()}, n_train: {n_train}.')
             print('Difference could be due to missing values.')
-        n_d_subsam = n_d.min() * cnew_dict['share_forest_sample']
-        if c_dict['n_min_min'] is None:
-            c_dict['n_min_min'] = cnew_dict['n_min_min'] = -1
+        # Feature preselection
+        cnew_dict['fs_yes'] = bool(c_dict['fs_yes'])
+        if c_dict['fs_rf_threshold'] is None or c_dict['fs_rf_threshold'] <= 0:
+            cnew_dict['fs_rf_threshold'] = 0.0001
+        cnew_dict['fs_other_sample'] = not c_dict['fs_other_sample'] is False
+        if c_dict['fs_other_sample_share'] is None or not (
+               0 <= c_dict['fs_other_sample_share'] <= 0.5):
+            cnew_dict['fs_other_sample_share'] = 0.33
+        if cnew_dict['fs_other_sample'] is False or (
+                cnew_dict['fs_yes'] is False):
+            cnew_dict['fs_other_sample_share'] = 0
+
+        # size of subsampling samples         n/2: size of forest sample
+        subsam_share_forest = mcf_ia.sub_size(
+            n_train, c_dict['subsample_factor_forest'], 0.67)
+        if c_dict['subsample_factor_eval'] is None:
+            c_dict['subsample_factor_eval'] = True  # Default
+        if c_dict['subsample_factor_eval'] is False:
+            c_dict['subsample_factor_eval'] = 1000000000
+        if c_dict['subsample_factor_eval'] is True:
+            c_dict['subsample_factor_eval'] = 2
+        if c_dict['subsample_factor_eval'] < 0.01:
+            c_dict['subsample_factor_eval'] = 1000000000
+        subsam_share_eval = min(
+            subsam_share_forest * c_dict['subsample_factor_eval'], 1)
+
+        n_d_subsam = (n_d.min() * cnew_dict['share_forest_sample']
+                      * subsam_share_forest)
+        if cnew_dict['fs_yes'] and cnew_dict['fs_other_sample']:
+            n_d_subsam *= (1 - cnew_dict['fs_other_sample_share'])
+        # Minimum leaf size
+        if c_dict['n_min_min'] is None or c_dict['n_min_min'] < 1:
+            cnew_dict['n_min_min'] = (max(round((n_d_subsam**0.4) / 6), 2)
+                                      * len(d_values))
         else:
-            cnew_dict['n_min_min'] = round(c_dict['n_min_min'])   # grid n_min
-        if cnew_dict['n_min_min'] < 1:
-            if cnew_dict['n_min_min'] == -1:
-                cnew_dict['n_min_min'] = round((n_d_subsam**0.4)/10)
-                if cnew_dict['n_min_min'] < 5:
-                    cnew_dict['n_min_min'] = 5
-            else:
-                cnew_dict['n_min_min'] = round((n_d_subsam**0.4)/20)
-                if cnew_dict['n_min_min'] < 3:
-                    cnew_dict['n_min_min'] = 3
+            cnew_dict['n_min_min'] = round(c_dict['n_min_min'])
+        if c_dict['n_min_max'] is None or c_dict['n_min_max'] < 1:
+            cnew_dict['n_min_max'] = (max(round(n_d_subsam**0.5 / 6), 4)
+                                      * len(d_values))
+        else:
+            cnew_dict['n_min_max'] = round(c_dict['n_min_max'])
+        cnew_dict['n_min_max'] = max(cnew_dict['n_min_min'],
+                                     cnew_dict['n_min_max'])
         if cnew_dict['d_type'] == 'discrete':
-            if cnew_dict['n_min_min'] < len(d_values):
-                cnew_dict['n_min_min'] = len(d_values)
-                text_to_print = ('Required to have support. Smallest ',
+            if c_dict['n_min_treat'] is None or c_dict['n_min_treat'] < 1:
+                cnew_dict['n_min_treat'] = max(round(
+                    (cnew_dict['n_min_min'] + cnew_dict['n_min_max']) / 2
+                    / len(d_values) / 4), 2)
+            else:
+                cnew_dict['n_min_treat'] = round(c_dict['n_min_treat'])
+            if cnew_dict['n_min_min'] < cnew_dict['n_min_treat'
+                                                  ] * len(d_values):
+                cnew_dict['n_min_min'] = cnew_dict['n_min_treat'
+                                                   ] * len(d_values)
+                text_to_print = ('Minimum leaf size adjusted. Smallest ',
                                  ' leafsize set to: ', cnew_dict['n_min_min'])
             else:
                 text_to_print = None
-        if c_dict['n_min_max'] is None:
-            c_dict['n_min_max'] = cnew_dict['n_min_max'] = -1
         else:
-            cnew_dict['n_min_max'] = round(c_dict['n_min_max'])
-        if cnew_dict['n_min_max'] < 1:
-            if cnew_dict['n_min_max'] == -1:
-                cnew_dict['n_min_max'] = round(math.sqrt(n_d_subsam) / 5)
-                if cnew_dict['n_min_max'] < 5:
-                    cnew_dict['n_min_max'] = 5
-            else:
-                cnew_dict['n_min_max'] = round(math.sqrt(n_d_subsam) / 10)
-                if cnew_dict['n_min_max'] < 3:
-                    cnew_dict['n_min_max'] = 3
-        if c_dict['n_min_grid'] is None:
-            cnew_dict['n_min_grid'] = 0
+            cnew_dict['n_min_treat'] = 0
+        if c_dict['n_min_grid'] is None or c_dict['n_min_grid'] < 1:
+            cnew_dict['n_min_grid'] = 1
         else:
             cnew_dict['n_min_grid'] = round(c_dict['n_min_grid'])
-        if cnew_dict['n_min_grid'] < 1:
-            cnew_dict['n_min_grid'] = 1
         if cnew_dict['n_min_min'] == cnew_dict['n_min_max']:
             cnew_dict['n_min_grid'] = 1
-        elif cnew_dict['n_min_max'] < cnew_dict['n_min_min']:
-            cnew_dict['n_min_grid'] = 1
-            cnew_dict['n_min_max'] = cnew_dict['n_min_min']
         if cnew_dict['n_min_grid'] == 1:
-            n_min = round(cnew_dict['n_min_min'])
-        elif cnew_dict['n_min_grid'] == 2:
-            n_min = np.hstack((cnew_dict['n_min_min'], cnew_dict['n_min_max']))
+            n_min = round(
+                (cnew_dict['n_min_min'] + cnew_dict['n_min_max']) / 2)
+            cnew_dict['n_min_min'] = cnew_dict['n_min_max'] = n_min
         else:
-            n_min = np.linspace(cnew_dict['n_min_min'], cnew_dict['n_min_max'],
-                                cnew_dict['n_min_grid'])
-            n_min = list(np.unique(np.round(n_min)))
-            cnew_dict['n_min_grid'] = len(n_min)
+            if cnew_dict['n_min_grid'] == 2:
+                n_min = np.hstack((cnew_dict['n_min_min'],
+                                   cnew_dict['n_min_max']))
+            else:
+                n_min = np.linspace(cnew_dict['n_min_min'],
+                                    cnew_dict['n_min_max'],
+                                    cnew_dict['n_min_grid'])
+            cnew_dict['n_min_min'] = n_min[0]
+            cnew_dict['n_min_max'] = n_min[-1]
+        n_min = list(np.unique(np.round(n_min)))
+        cnew_dict['n_min_grid'] = len(n_min)
+
         # Select grid for number of parameters
-        if c_dict['m_min_share'] is None:
-            c_dict['m_min_share'] = -1
-        if (c_dict['m_min_share'] <= 0) or (c_dict['m_min_share'] > 1):
-            cnew_dict['m_min_share'] = 0.1   # Hastie et al suggest p/3,sqrt(p)
-            if c_dict['m_min_share'] == -2:  # around page 588
-                cnew_dict['m_min_share'] = 0.2
-        if c_dict['m_max_share'] is None:
-            c_dict['m_max_share'] = -1
-        if (c_dict['m_max_share'] <= 0) or (c_dict['m_max_share'] > 1):
-            cnew_dict['m_max_share'] = 0.66
-            if c_dict['m_max_share'] == -2:
-                cnew_dict['m_max_share'] = 0.8
-        # Instead of selecting fixed number, select mean of Poisson distrib.
+        if (c_dict['m_share_min'] is None or c_dict['m_share_min'] <= 0
+                or c_dict['m_share_min'] > 1):
+            cnew_dict['m_share_min'] = 0.1
+        if (c_dict['m_share_max'] is None or c_dict['m_share_max'] <= 0
+                or c_dict['m_share_max'] > 1):
+            cnew_dict['m_share_max'] = 0.6
         if c_dict['m_random_poisson'] is False:
             cnew_dict['m_random_poisson'] = False
+            cnew_dict['m_random_poisson_min'] = 1000000
         else:
             cnew_dict['m_random_poisson'] = True
-        if c_dict['m_grid'] is None:
-            c_dict['m_grid'] = -1
-        if c_dict['m_grid'] < 1:  # values of grid for m
-            cnew_dict['m_grid'] = 2
+            if c_dict['m_random_poisson'] is True:
+                cnew_dict['m_random_poisson_min'] = 0
+            else:
+                cnew_dict['m_random_poisson_min'] = 10
+        if c_dict['m_grid'] is None or c_dict['m_grid'] < 1:
+            cnew_dict['m_grid'] = 1        # values of grid for m
         else:
             cnew_dict['m_grid'] = round(c_dict['m_grid'])
+
         if c_dict['max_cats_cont_vars'] is None:
             c_dict['max_cats_cont_vars'] = -1
         cnew_dict['max_cats_cont_vars'] = (
@@ -359,56 +382,18 @@ def get_controls(c_dict, v_dict):
             c_dict['random_thresholds'] = -1
         if c_dict['random_thresholds'] < 0:     # Saves computation time
             cnew_dict['random_thresholds'] = round(4 + n_train**0.2)
-        # Feature preselection
-        cnew_dict['fs_yes'] = bool(c_dict['fs_yes'])
-        if c_dict['fs_rf_threshold'] is None:
-            c_dict['fs_rf_threshold'] = -1
-        if c_dict['fs_rf_threshold'] <= 0:
-            cnew_dict['fs_rf_threshold'] = 0
-        cnew_dict['fs_other_sample'] = not c_dict['fs_other_sample'] is False
-        if c_dict['fs_other_sample_share'] is None:
-            c_dict['fs_other_sample_share'] = -1
-        if (c_dict['fs_other_sample_share'] < 0) or (
-                c_dict['fs_other_sample_share'] > 0.5):
-            cnew_dict['fs_other_sample_share'] = 0.2
-        if cnew_dict['fs_other_sample'] is False or (
-                cnew_dict['fs_yes'] is False):
-            cnew_dict['fs_other_sample_share'] = 0
 
-        # size of subsampling samples         n/2: size of forest sample
-        subsam_share_forest = mcf_ia.sub_size(
-            n_train, c_dict['subsample_factor_forest'], 0.67)
-        if c_dict['subsample_factor_eval'] is None:
-            c_dict['subsample_factor_eval'] = False  # Default
-        if c_dict['subsample_factor_eval'] is False:
-            c_dict['subsample_factor_eval'] = 1000000000
-        if c_dict['subsample_factor_eval'] is True:
-            c_dict['subsample_factor_eval'] = 1
-        if c_dict['subsample_factor_eval'] < 0.01:
-            c_dict['subsample_factor_eval'] = 1000000000
-        subsam_share_eval = mcf_ia.sub_size(
-            n_train, c_dict['subsample_factor_eval'], 1)
-        # Define method to be used later on
-        if c_dict['mce_vart'] is None:
-            c_dict['mce_vart'] = -1
-        if (c_dict['mce_vart'] == 1) or (c_dict['mce_vart'] == -1):
-            mtot = 1
-            mtot_no_mce = 0        # MSE + MCE rule
-            estimator_str = 'MSE & MCE'
-        elif c_dict['mce_vart'] == 2:
-            mtot = 2
-            mtot_no_mce = 1        # -Var(treatment effect) rule
-            estimator_str = '-Var(effect)'
-        elif c_dict['mce_vart'] == 0:
-            mtot = 3
-            mtot_no_mce = 1        # MSE rule
-            estimator_str = 'MSE'
-        elif c_dict['mce_vart'] == 3:
-            mtot = 4               # MSE + MCE rule or penalty function rule
-            mtot_no_mce = 0        # (randomly decided)
+        if c_dict['mce_vart'] is None or c_dict['mce_vart'] == 1:
+            mtot, mtot_no_mce, estimator_str = 1, 0, 'MSE & MCE'    # MSE + MCE
+        elif c_dict['mce_vart'] == 2:                  # -Var(treatment effect)
+            mtot, mtot_no_mce, estimator_str = 2, 1, '-Var(effect)'
+        elif c_dict['mce_vart'] == 0:                             # MSE rule
+            mtot, mtot_no_mce, estimator_str = 3, 1, 'MSE'
+        elif c_dict['mce_vart'] == 3:  # MSE+MCE rule or penalty function rule
+            mtot, mtot_no_mce = 4, 0                      # (randomly decided)
             estimator_str = 'MSE,MCE or penalty (random)'
         else:
-            raise Exception('Inconsistent MTOT definition of  MCE_VarT.')
+            raise ValueError('Inconsistent MTOT definition of  MCE_VarT.')
         if c_dict['mtot_p_diff_penalty'] is None:
             c_dict['mtot_p_diff_penalty'] = -1
         if c_dict['mtot_p_diff_penalty'] == 0:
@@ -488,7 +473,7 @@ def get_controls(c_dict, v_dict):
         # Choice based sampling (oversampling of treatments)
         if c_dict['choice_based_yes'] is True:
             err_txt = 'No choice based sample with continuous treatments.'
-            assert ['d_type'] == 'discrete', err_txt
+            assert cnew_dict['d_type'] == 'discrete', err_txt
             cnew_dict['choice_based_yes'] = True
             err_t = ('Choice based sampling. Rows in choice probabilities do'
                      + ' not correspond to number of treatments.')
@@ -506,9 +491,8 @@ def get_controls(c_dict, v_dict):
         else:
             cnew_dict['choice_based_yes'] = False
             cnew_dict['choice_based_probs'] = 1
-        if c_dict['max_weight_share'] is None:
-            c_dict['max_weight_share'] = -1
-        if c_dict['max_weight_share'] <= 0:
+        if (c_dict['max_weight_share'] is None
+                or c_dict['max_weight_share'] <= 0):
             cnew_dict['max_weight_share'] = 0.05
         # Parameters for the variance estimation
         if c_dict['cond_var'] is False:    # False: variance est. uses Var(wY)
@@ -516,22 +500,14 @@ def get_controls(c_dict, v_dict):
         else:
             cnew_dict['cond_var'] = True
         cnew_dict['knn'] = c_dict['knn'] is not False
-        if c_dict['knn_min_k'] is None:
-            c_dict['knn_min_k'] = -1
-        if c_dict['knn_min_k'] < 0:     # minimum number of neighbours in k-NN
-            cnew_dict['knn_min_k'] = 10
-        if c_dict['knn_const'] is None:
-            c_dict['knn_const'] = -1
-        if c_dict['knn_const'] < 0:  # k: const. in # of neighbour estimation
-            cnew_dict['knn_const'] = 1
-        if c_dict['nw_bandw'] is None:
-            c_dict['nw_bandw'] = -1
-        if c_dict['nw_bandw'] < 0:   # bandwidth for NW estimat. multiplier of
-            cnew_dict['nw_bandw'] = 1  # silverman's optimal bandwidth
-        if c_dict['nw_kern'] is None:
-            c_dict['nw_kern'] = -1
-        if c_dict['nw_kern'] != 2:  # kernel for NW: 1: Epanechikov 2: Normal
-            cnew_dict['nw_kern'] = 1
+        if c_dict['knn_min_k'] is None or c_dict['knn_min_k'] < 0:
+            cnew_dict['knn_min_k'] = 10  # minimum number of neighbours in k-NN
+        if c_dict['knn_const'] is None or c_dict['knn_const'] < 0:
+            cnew_dict['knn_const'] = 1  # k: const. in # of neighbour estimat.
+        if c_dict['nw_bandw'] is None or c_dict['nw_bandw'] < 0:  # multiplier
+            cnew_dict['nw_bandw'] = 1      # of silverman's optimal bandwidth
+        if c_dict['nw_kern'] is None or c_dict['nw_kern'] != 2:  # kernel for
+            cnew_dict['nw_kern'] = 1          # NW: 1: Epanechikov 2: Normal
         bnr = 199
         if c_dict['se_boot_ate'] is None:
             c_dict['se_boot_ate'] = bnr if cnew_dict['cluster_std'] else False
@@ -569,9 +545,8 @@ def get_controls(c_dict, v_dict):
             cnew_dict['gatet_flag'] = cnew_dict['atet_flag'] = True
         else:
             cnew_dict['gatet_flag'] = False
-        if c_dict['gmate_sample_share'] is None:
-            c_dict['gmate_sample_share'] = -1
-        if c_dict['gmate_sample_share'] <= 0:
+        if (c_dict['gmate_sample_share'] is None
+                or c_dict['gmate_sample_share'] <= 0):
             cnew_dict['gmate_sample_share'] = (
                 1 if n_pred < 1000
                 else (1000 + ((n_pred-1000) ** 0.75)) / n_pred)
@@ -591,13 +566,17 @@ def get_controls(c_dict, v_dict):
         cnew_dict['sgates_no_evaluation_points'] = (
             50 if c_dict['sgates_no_evaluation_points'] < 2
             else round(c_dict['sgates_no_evaluation_points']))
+        if c_dict['gates_minus_previous']:
+            cnew_dict['gates_minus_previous'] = True
+        else:
+            cnew_dict['gates_minus_previous'] = False
         if c_dict['iate_flag'] is False:
             cnew_dict['iate_flag'] = cnew_dict['iate_se_flag'] = False
             c_dict['post_est_stats'] = False
         else:
             cnew_dict['iate_flag'] = True
             cnew_dict['iate_se_flag'] = not c_dict['iate_se_flag'] is False
-        cnew_dict['effiate_flag'] = not c_dict['effiate_flag'] is False
+        cnew_dict['iate_eff_flag'] = not c_dict['iate_eff_flag'] is False
         # Post estimation parameters
         cnew_dict['post_est_stats'] = not c_dict['post_est_stats'] is False
         cnew_dict['bin_corr_yes'] = not c_dict['bin_corr_yes'] is False
@@ -655,8 +634,11 @@ def get_controls(c_dict, v_dict):
             cnew_dict['mp_type_weights'] = 1  # MP over obs, fast, lots of mem
         else:
             cnew_dict['mp_type_weights'] = 2
-        cnew_dict['weight_as_sparse'] = not (c_dict['weight_as_sparse']
-                                             is False)
+        cnew_dict['_weight_as_sparse'] = not (c_dict['_weight_as_sparse']
+                                              is False)
+        if not isinstance(c_dict['_weight_as_sparse_splits'], int):
+            cnew_dict['_weight_as_sparse_splits'] = None
+#  Is set in weight function because sample sizes are needed
         if c_dict['mp_weights_tree_batch'] is None:
             c_dict['mp_weights_tree_batch'] = -1
         cnew_dict['mp_weights_tree_batch'] = (
@@ -694,20 +676,19 @@ def get_controls(c_dict, v_dict):
         c_dict['common_support'] = -1
     if c_dict['common_support'] != 0 and c_dict['common_support'] != 2:
         cnew_dict['common_support'] = 1      # Default values
+    cnew_dict['support_cf'] = not c_dict['support_cf'] is False
     if cnew_dict['common_support'] > 0:
         if c_dict['support_quantil'] is None:
             c_dict['support_quantil'] = -1
-        if c_dict['support_quantil'] < 0 or c_dict['support_quantil'] > 1:
+        if not 0 <= c_dict['support_quantil'] <= 1:
             cnew_dict['support_quantil'] = 1
         if c_dict['support_min_p'] is None:
             c_dict['support_min_p'] = -1
-        if c_dict['support_min_p'] < 0 or c_dict['support_min_p'] > 0.5:
+        if not 0 <= c_dict['support_min_p'] <= 0.5:
             cnew_dict['support_min_p'] = 0.01
-    if c_dict['support_max_del_train'] is None:
-        c_dict['support_max_del_train'] = -1
-    if not 0 < c_dict['support_max_del_train'] <= 1:
+    if (c_dict['support_max_del_train'] is None
+            or not 0 < c_dict['support_max_del_train'] <= 1):
         cnew_dict['support_max_del_train'] = 0.33
-    cnew_dict['mp_with_ray'] = not c_dict['mp_with_ray'] is False
     memory = psutil.virtual_memory()
     if c_dict['mp_ray_objstore_multiplier'] is None:
         c_dict['mp_ray_objstore_multiplier'] = -1
@@ -745,10 +726,8 @@ def get_controls(c_dict, v_dict):
         mem_object_store_3 = 3
         n_pred = 0
     mp_automatic = False
-    if c_dict['no_parallel'] is None:
-        c_dict['no_parallel'] = -1
-    if c_dict['no_parallel'] < -0.5:
-        cnew_dict['no_parallel'] = psutil.cpu_count(logical=True) - 2
+    if c_dict['no_parallel'] is None or c_dict['no_parallel'] < -0.5:
+        cnew_dict['no_parallel'] = round(psutil.cpu_count(logical=True)*0.8)
         mp_automatic = True
     elif -0.5 <= c_dict['no_parallel'] <= 1.5:
         cnew_dict['no_parallel'] = 1
@@ -756,15 +735,20 @@ def get_controls(c_dict, v_dict):
         cnew_dict['no_parallel'] = round(c_dict['no_parallel'])
     sys_share = 0.7 * getattr(psutil.virtual_memory(), 'percent') / 100
 
+    if c_dict['_ray_or_dask'] != 'dask':
+        cnew_dict['_ray_or_dask'] = 'ray'
+
     if c_dict['_mp_ray_del'] is None:
         cnew_dict['_mp_ray_del'] = ('refs',)
     else:
         possible_vals = ('refs', 'rest', 'none')
-        if isinstance(c_dict['_mp_ray_del'], (str, list)):
+        if isinstance(c_dict['_mp_ray_del'], str):
+            cnew_dict['_mp_ray_del'] = (c_dict['_mp_ray_del'],)
+        if isinstance(c_dict['_mp_ray_del'], list):
             cnew_dict['_mp_ray_del'] = tuple(c_dict['_mp_ray_del'])
         if len(cnew_dict['_mp_ray_del']) > 2:
             print(cnew_dict['_mp_ray_del'])
-            raise Exception('Too many parameters for _mp_ray_del')
+            raise ValueError('Too many parameters for _mp_ray_del')
         ass_err = '_mp_ray_del is no Tuple'
         assert isinstance(cnew_dict['_mp_ray_del'], tuple), ass_err
         ass_e = 'Wrong parameters for _mp_ray_del'
@@ -847,8 +831,11 @@ def get_controls(c_dict, v_dict):
         cnew_dict['ct_grid_nn'] = cnew_dict['ct_grid_nn_val'] = None
         cnew_dict['ct_grid_dr'] = cnew_dict['ct_grid_dr_val'] = None
     if c_dict['support_adjust_limits'] is None:
-        cnew_dict['support_adjust_limits'] = (no_of_treat - 2) * 0.05
-        err_txt = 'Negative common support adjustment factor not possible.'
+        if cnew_dict['support_cf']:
+            cnew_dict['support_adjust_limits'] = 0
+        else:
+            cnew_dict['support_adjust_limits'] = (no_of_treat - 2) * 0.05
+    err_txt = 'Negative common support adjustment factor not possible.'
     assert cnew_dict['support_adjust_limits'] >= 0, err_txt
     cn_add = {
         'temppfad': temppfad, 'pred_sample_with_pred': pred_sample_with_pred,
@@ -878,24 +865,24 @@ def get_controls(c_dict, v_dict):
     cnew_dict.update(cn_add)
     # Check for inconsistencies
     if cnew_dict['iate_flag'] is False or cnew_dict['pred_mcf'] is False:
-        cnew_dict['effiate_flag'] = False
-    if cnew_dict['effiate_flag'] and not cnew_dict['train_mcf']:
+        cnew_dict['iate_eff_flag'] = False
+    if cnew_dict['iate_eff_flag'] and not cnew_dict['train_mcf']:
         print('2nd round efficient IATEs are only computed if training is ',
               'activated. They will be disabled.')
-        cnew_dict['effiate_flag'] = False
+        cnew_dict['iate_eff_flag'] = False
     if c_dict['train_mcf']:
         if v_dict['y_name'] is None or v_dict['y_name'] == []:
-            raise Exception('y_name must be specified.')
+            raise ValueError('y_name must be specified.')
         if v_dict['d_name'] is None or v_dict['d_name'] == []:
-            raise Exception('d_name must be specified.')
+            raise ValueError('d_name must be specified.')
         if v_dict['y_tree_name'] is None or v_dict['y_tree_name'] == []:
             if isinstance(v_dict['y_name'], (list, tuple)):
                 v_dict['y_tree_name'] = [v_dict['y_name'][0]]
             else:
-                v_dict['y_tree_name'] = [v_dict['y_name']]        
+                v_dict['y_tree_name'] = [v_dict['y_name']]
     if (v_dict['x_name_ord'] is None or v_dict['x_name_ord'] == []) and (
             v_dict['x_name_unord'] is None or v_dict['x_name_unord'] == []):
-        raise Exception('x_name_ord or x_name_unord must be specified.')
+        raise ValueError('x_name_ord or x_name_unord must be specified.')
     vnew_dict = copy.deepcopy(v_dict)
     if cnew_dict['cluster_std'] or cnew_dict['panel']:
         ass_err = 'More than 1 name for cluster variable.'
@@ -935,7 +922,7 @@ def get_controls(c_dict, v_dict):
         vnew_dict['w_name'] = []
     else:
         if vnew_dict['w_name'] is None or vnew_dict['w_name'] == []:
-            raise Exception('No name for sample weights specified.')
+            raise ValueError('No name for sample weights specified.')
         vnew_dict['w_name'] = gp.cleaned_var_names(vnew_dict['w_name'])
     vnew_dict['id_name'] = gp.cleaned_var_names(vnew_dict['id_name'])
 
@@ -962,17 +949,15 @@ def get_controls(c_dict, v_dict):
         if cnew_dict['balancing_test_w']:
             x_name_remain = gp.cleaned_var_names(copy.deepcopy(
                 x_balance_name + x_name_remain))
-    if x_name_in_tree != []:
+    if x_name_in_tree:
         x_name_remain = gp.cleaned_var_names(copy.deepcopy(
             x_name_in_tree + x_name_remain))
         x_name = gp.cleaned_var_names(copy.deepcopy(x_name_in_tree + x_name))
     if not ((name_ordered == []) or (name_unordered == [])):
         if any(value for value in name_ordered if value in name_unordered):
-            raise Exception('Remove overlap in ordered + unordered variables')
-    v_dict['z_name_mgate'] = gp.cleaned_var_names(
-        v_dict['z_name_mgate'])
-    v_dict['z_name_amgate'] = gp.cleaned_var_names(
-        v_dict['z_name_amgate'])
+            raise ValueError('Remove overlap in ordered + unordered variables')
+    v_dict['z_name_mgate'] = gp.cleaned_var_names(v_dict['z_name_mgate'])
+    v_dict['z_name_amgate'] = gp.cleaned_var_names(v_dict['z_name_amgate'])
     vnew_dict['z_name_mgate'] = list(
         set(v_dict['z_name_mgate']).intersection(x_name))
     vnew_dict['z_name_amgate'] = list(
@@ -1030,7 +1015,7 @@ def get_controls(c_dict, v_dict):
                        + ' prediction file, WHICH IS MISSING!')
             assert not cnew_dict['choice_based_yes'], err_txt
 
-    if  cnew_dict['fs_yes']:
+    if cnew_dict['fs_yes']:
         cnew_dict['l_centering_uncenter'] = False
     vn_add = {
         'x_name_remain': x_name_remain, 'x_name': x_name, 'z_name': z_name,
