@@ -5,6 +5,7 @@ Contains the functions needed for the weight computation of MCF
 -*- coding: utf-8 -*-
 """
 from concurrent import futures
+from dask.distributed import Client, as_completed
 
 import numpy as np
 import pandas as pd
@@ -42,8 +43,87 @@ def get_weights_mp(forest, x_file, y_file, v_dict, c_dict, x_name,
     if c_dict['with_output'] and c_dict['verbose']:
         print('\nObtaining weights from estimated forest')
     data_x = pd.read_csv(x_file)
-    x_dat = data_x[x_name].to_numpy()
+    x_dat_all = data_x[x_name].to_numpy()
     del data_x
+    if c_dict['_weight_as_sparse_splits'] is None:
+        if len(x_dat_all) < 5000:
+            c_dict['_weight_as_sparse_splits'] = 1
+        else:
+            n_y = len(pd.read_csv(y_file))
+            c_dict['_weight_as_sparse_splits'] = round(n_y * len(x_dat_all)
+                                                       / (20000 * 20000))
+        if c_dict['_weight_as_sparse_splits'] < 1:
+            c_dict['_weight_as_sparse_splits'] = 1
+    if c_dict['_weight_as_sparse_splits'] == 1 or not c_dict[
+            '_weight_as_sparse']:
+        weights, y_dat, x_bala, cl_dat, w_dat = get_weights_mp_inner(
+            forest, x_dat_all, y_file, v_dict, c_dict, regrf=regrf)
+        return weights, y_dat, x_bala, cl_dat, w_dat
+
+    base_file = c_dict['temppfad'] + '/' + 'temp_weights'
+    x_dat_list = np.array_split(x_dat_all, c_dict['_weight_as_sparse_splits'],
+                                axis=0)
+    no_x_bala_return = True
+    f_name = []
+    for idx, x_dat in enumerate(x_dat_list):
+        if c_dict['with_output'] and c_dict['verbose']:
+            print('\nWeights computation with additional splitting. ',
+                  f'Mastersplit {idx} ({len(x_dat_list)})')
+        if idx == len(x_dat_list) - 1:  # Last iteration, get x_bala
+            no_x_bala_return = False
+        weights_i, y_dat, x_bala, cl_dat, w_dat = get_weights_mp_inner(
+            forest, x_dat, y_file, v_dict, c_dict, regrf=regrf,
+            no_x_bala_return=no_x_bala_return)
+        f_name_i = []
+        if c_dict['_weight_as_sparse_splits'] > 1:
+            elem_per_chunk = len(weights_i)
+            for d_idx, weight_di in enumerate(weights_i):
+                f_name_d_idx = base_file + str(idx) + '_' + str(d_idx) + '.npz'
+                gp.delete_file_if_exists(f_name_d_idx)
+                sparse.save_npz(f_name_d_idx, weight_di)
+                f_name_i.append(f_name_d_idx)
+            f_name.append(f_name_i)
+            del weights_i
+        else:
+            weights = weights_i
+    del x_dat_list, x_dat_all
+    if c_dict['_weight_as_sparse_splits'] > 1:
+        weights = []
+        for d_idx in range(elem_per_chunk):
+            w_list = []
+            for idx in range(c_dict['_weight_as_sparse_splits']):
+                file_to_load = f_name[idx][d_idx]
+                w_list.append(sparse.load_npz(file_to_load))
+            w_all = sparse.vstack(w_list)
+            weights.append(w_all)
+        del w_all, w_list
+    return weights, y_dat, x_bala, cl_dat, w_dat
+
+
+def get_weights_mp_inner(forest, x_dat, y_file, v_dict, c_dict, regrf=False,
+                         no_x_bala_return=False):
+    """Get weights for obs in pred_data & outcome and cluster from y_data.
+
+    Parameters
+    ----------
+    forest : Tuple of lists. Node_table.
+    x_dat : Numpy array. X-data to make predictions for.
+    y_file : String. csv-file with outcome data.
+    v_dict :  Dict. Variables.
+    c_dict :  Dict. Parameters.
+    regrf : Bool. Honest regression Random Forest. Default is False.
+    no_x_bala_return: Bool. Do not return X for balancing tests despite
+                            c_dict['balancing_test_w'] being True.
+                            Default is False.
+
+    Returns
+    -------
+    weights : Tuple of lists (N_pred x 1 (x no_of_treat + 1).
+    y_data : N_y x number of outcomes-Numpy array. Outcome variables.
+    x_bala : N_y x number of balancing-vars-Numpy array. Balancing test vars.
+    cl_dat : N_y x 1 Numpy array. Cluster number.
+    w_dat: N_y x 1 Numpy array. Sampling weights (if used).
+    """
     data_y = pd.read_csv(y_file)
     y_dat = data_y[v_dict['y_name']].to_numpy()
     d_dat = None if regrf else data_y[v_dict['d_name']].to_numpy()
@@ -55,7 +135,7 @@ def get_weights_mp(forest, x_file, y_file, v_dict, c_dict, x_name,
     w_dat = data_y[v_dict['w_name']].to_numpy() if c_dict['w_yes'] else 0
     x_bala = 0
     if not regrf:
-        if c_dict['balancing_test_w']:
+        if c_dict['balancing_test_w'] and not no_x_bala_return:
             x_bala = data_y[v_dict['x_balance_name']].to_numpy()
     del data_y
     empty_leaf_counter = merge_leaf_counter = 0
@@ -69,7 +149,7 @@ def get_weights_mp(forest, x_file, y_file, v_dict, c_dict, x_name,
             maxworkers = c_dict['no_parallel']
     if c_dict['with_output'] and c_dict['verbose']:
         print('Number of parallel processes: ', maxworkers)
-    if maxworkers == 1 or c_dict['mp_with_ray']:
+    if maxworkers == 1 or c_dict['_ray_or_dask'] == 'ray':
         mp_over_boots = False
     else:
         mp_over_boots = (False if regrf
@@ -86,7 +166,7 @@ def get_weights_mp(forest, x_file, y_file, v_dict, c_dict, x_name,
                 mp_over_boots, maxworkers, False)
             if c_dict['with_output'] and c_dict['verbose']:
                 gp.share_completed(idx+1, n_x)
-            if c_dict['weight_as_sparse']:
+            if c_dict['_weight_as_sparse']:
                 for d_idx in range(no_of_treat):
                     indices = results_fut_idx[1][d_idx][0]
                     weights_obs = results_fut_idx[1][d_idx][1]
@@ -97,7 +177,7 @@ def get_weights_mp(forest, x_file, y_file, v_dict, c_dict, x_name,
                 weights[idx] = results_fut_idx[1]
             empty_leaf_counter += results_fut_idx[2]
             merge_leaf_counter += results_fut_idx[3]
-        if c_dict['weight_as_sparse']:
+        if c_dict['_weight_as_sparse']:
             weights = weights_to_csr(weights, no_of_treat, regrf)
     else:
         no_of_splits_i, max_size_i = maxworkers, 1000
@@ -114,7 +194,7 @@ def get_weights_mp(forest, x_file, y_file, v_dict, c_dict, x_name,
                   f' {n_x / no_of_splits_i:5.2f}')
         all_idx_split = np.array_split(range(n_x), no_of_splits_i)
         split_forest = False
-        if not c_dict['mp_with_ray']:
+        if c_dict['_ray_or_dask'] != 'ray':
             if c_dict['mp_weights_tree_batch'] > 1:  # User def. # of batches
                 no_of_boot_splits = c_dict['mp_weights_tree_batch']
                 split_forest = True
@@ -139,7 +219,7 @@ def get_weights_mp(forest, x_file, y_file, v_dict, c_dict, x_name,
                       f' {c_dict["boot"]/no_of_boot_splits:5.2f}')
         else:
             boot_indx_list = range(1)
-        if (not c_dict['weight_as_sparse']) and split_forest:
+        if (not c_dict['_weight_as_sparse']) and split_forest:
             if c_dict['with_output'] and c_dict['verbose']:
                 print('XXXXXXXXDANGERXXXX' * 3)
                 print('Bootstrap splitting requires using sparse',
@@ -164,9 +244,9 @@ def get_weights_mp(forest, x_file, y_file, v_dict, c_dict, x_name,
                     print(f'Boot Chunk {b_i+1:2} of {no_of_boot_splits:2}')
                     gp_sys.memory_statistics()
             else:
-                if not c_dict['mp_with_ray']:
+                if c_dict['_ray_or_dask'] != 'ray':
                     forest_temp = forest
-            if c_dict['mp_with_ray']:
+            if c_dict['_ray_or_dask'] == 'ray':
                 if c_dict['mem_object_store_3'] is None:
                     if not ray.is_initialized():
                         ray.init(num_cpus=maxworkers, include_dashboard=False)
@@ -179,8 +259,7 @@ def get_weights_mp(forest, x_file, y_file, v_dict, c_dict, x_name,
                         print("Size of Ray Object Store: ",
                               round(c_dict['mem_object_store_3']/(1024*1024)),
                               " MB")
-                x_dat_ref = ray.put(x_dat)
-                forest_ref = ray.put(forest)
+                x_dat_ref, forest_ref = ray.put(x_dat), ray.put(forest)
                 still_running = [ray_weights_many_obs_i.remote(
                     idx_list, n_y, forest_ref, x_dat_ref, d_dat, c_dict, regrf,
                     split_forest) for idx_list in all_idx_split]
@@ -199,7 +278,7 @@ def get_weights_mp(forest, x_file, y_file, v_dict, c_dict, x_name,
                                       end='')
                             jdx += 1
                         for idx, val_list in enumerate(results_fut_idx[0]):
-                            if c_dict['weight_as_sparse']:
+                            if c_dict['_weight_as_sparse']:
                                 for d_idx in range(no_of_treat):
                                     indices = results_fut_idx[1][idx][d_idx][0]
                                     weights_obs = results_fut_idx[1][idx][d_idx
@@ -220,6 +299,40 @@ def get_weights_mp(forest, x_file, y_file, v_dict, c_dict, x_name,
                     del finished_res, finished
                 if c_dict['_mp_ray_shutdown']:
                     ray.shutdown()
+            elif c_dict['_ray_or_dask'] == 'dask':
+                with Client(n_workers=maxworkers) as clt:
+                    x_dat_ref = clt.scatter(x_dat)
+                    d_dat_ref = clt.scatter(d_dat)
+                    forest_temp_ref = clt.scatter(forest_temp)
+                    ret_fut = [clt.submit(
+                        weights_many_obs_i, idx_list, n_y, forest_temp_ref,
+                        x_dat_ref, d_dat_ref, c_dict, regrf, split_forest)
+                        for idx_list in all_idx_split]
+                    jdx = 0
+                    for _, res in as_completed(ret_fut, with_results=True):
+                        if c_dict['with_output'] and c_dict['verbose']:
+                            if jdx == 0:
+                                print()
+                                print(f'   Obs chunk {jdx+1:2}',
+                                      f' ({no_of_splits_i:2})', end='')
+                            else:
+                                print(f' {jdx+1:2} ({no_of_splits_i:2})',
+                                      end='')
+                            jdx += 1
+                        idx_list = res[0]
+                        for idx, val_list in enumerate(idx_list):
+                            if c_dict['_weight_as_sparse']:
+                                for d_idx in range(no_of_treat):
+                                    indices = res[1][idx][d_idx][0]
+                                    weights_obs = res[1][idx][d_idx][1]
+                                    weights[d_idx][val_list, indices
+                                                   ] = weights_obs
+                                    if regrf:
+                                        break
+                            else:
+                                weights[val_list] = res[1][idx]
+                        empty_leaf_counter += res[2]
+                        merge_leaf_counter += res[3]
             else:
                 with futures.ProcessPoolExecutor(max_workers=maxworkers
                                                  ) as fpp:
@@ -241,7 +354,7 @@ def get_weights_mp(forest, x_file, y_file, v_dict, c_dict, x_name,
                                       end='')
                         idx_list = results_fut_idx[0]
                         for idx, val_list in enumerate(idx_list):
-                            if c_dict['weight_as_sparse']:
+                            if c_dict['_weight_as_sparse']:
                                 for d_idx in range(no_of_treat):
                                     indices = results_fut_idx[1][idx][d_idx][0]
                                     weights_obs = results_fut_idx[1][idx][d_idx
@@ -254,7 +367,7 @@ def get_weights_mp(forest, x_file, y_file, v_dict, c_dict, x_name,
                                 weights[val_list] = results_fut_idx[1][idx]
                         empty_leaf_counter += results_fut_idx[2]
                         merge_leaf_counter += results_fut_idx[3]
-            if c_dict['weight_as_sparse']:
+            if c_dict['_weight_as_sparse']:
                 weights = weights_to_csr(weights, no_of_treat, regrf)
             if split_forest:
                 if b_i == 0:  # only in case of splitted forests
@@ -267,12 +380,12 @@ def get_weights_mp(forest, x_file, y_file, v_dict, c_dict, x_name,
                 if c_dict['with_output'] and c_dict['verbose']:
                     print()
                     mcf_gp.print_size_weight_matrix(
-                        weights_all, c_dict['weight_as_sparse'], no_of_treat)
+                        weights_all, c_dict['_weight_as_sparse'], no_of_treat)
                     gp_sys.memory_statistics()
     if split_forest:
         c_dict['boot'] = total_bootstraps
         weights = normalize_weights(weights_all, no_of_treat, regrf,
-                                    c_dict['weight_as_sparse'], n_x)
+                                    c_dict['_weight_as_sparse'], n_x)
     weights = tuple(weights)
     if (((empty_leaf_counter > 0) or (merge_leaf_counter > 0))
             and c_dict['with_output']) and c_dict['verbose']:
@@ -318,7 +431,7 @@ def initialise_weights(c_dict, n_x, n_y, regrf):
     no_of_treat = (len(c_dict['ct_grid_w_val'])
                    if c_dict['d_type'] == 'continuous'
                    else c_dict['no_of_treat'])
-    if c_dict['weight_as_sparse']:
+    if c_dict['_weight_as_sparse']:
         weights = []
         for _ in range(no_of_treat):
             weights.append(sparse.lil_matrix((n_x, n_y), dtype=np.float32))
@@ -496,8 +609,9 @@ def weights_obs_i_inside_boot(forest_b, x_dat_i, regrf, n_y, no_of_treat,
                 # We need to collect information over various leafs for the 0
                 # For the other leaves, the leaves are treatment specific
                 leaf_0_complete, leaf_pos_complete = False, True
-                for jdx, _ in enumerate(fb_lid_14_list):  # Zuerst 0 einsammeln
-                    fb_lid_14 = fb_lid_14_list[jdx]
+                # Zuerst 0 einsammeln
+                for jdx, fb_lid_14 in enumerate(fb_lid_14_list):
+                    # fb_lid_14 = fb_lid_14_list[jdx]
                     d_ib = d_dat[fb_lid_14].reshape(-1)  # view
                     indices_ibj_0 = d_ib < 1e-15
                     indices_ibj_pos = d_ib >= 1e-15
